/**
 * External dependencies
 */
import classnames from "classnames";
import { map, isUndefined } from "lodash";

/**
 * WordPress dependencies
 */
import domReady from "@wordpress/dom-ready";
import { render } from "@wordpress/element";
import { __ } from "@wordpress/i18n";
import { Spinner, Notice } from "@wordpress/components";
import { useReducer, useMemo, useState } from "@wordpress/element";
import { getBlockTypes } from "@wordpress/blocks";
import { useSelect } from "@wordpress/data";
import { useEntityRecords } from "@wordpress/core-data";
import { registerCoreBlocks } from "@wordpress/block-library";

/**
 * Internal dependencies
 */
import { useLocalStorage } from "sdk/utils";
import { store as boldblocksDataStore } from "../store";
import { store as blockLibraryDataStore } from "./store";
import {
  LibraryContext,
  initialState,
  reducer,
  dispatchLibraryState,
  useLibraryContent,
  cachedLibraryStateKey,
  getBlockLibraryURL,
  importData,
} from "./data";
import {
  getIsResolving,
  getHasFinishedResolution,
  userCanManagePlugins,
  getEditPostURL,
} from "../utils";
import { PageStyled } from "./components/styles";
import { default as TemplateGridControl } from "../components/template-modal/grid-control";
import { KeywordsControl } from "../components/template-modal/keywords-control";
import { SortControl } from "../components/template-modal/sort-control";
import { HeaderActionsControl } from "../components/template-modal/header-actions-control";
import { TemplateItemControl } from "./components/grid-control";

// Register core blocks.
registerCoreBlocks();

export const BlockLibraryPage = ({ className }) => {
  const contentType = "block";
  const { getPlugins } = useSelect((select) => select(boldblocksDataStore), []);
  const { getBlocks, getBlockKeywords, getBlockLibraryState } = useSelect(
    (select) => select(blockLibraryDataStore),
    []
  );

  // Query plugins data
  const canManagePlugins = userCanManagePlugins();

  let plugins = getPlugins();
  const isLoadingPlugins = getIsResolving("getPlugins");
  let isFinishedLoadingPlugins = getHasFinishedResolution("getPlugins");
  if (!isUndefined(canManagePlugins) && !canManagePlugins) {
    isFinishedLoadingPlugins = true;
  }

  // Query blocks
  const blockData = getBlocks();
  const isLoadingBlocks = getIsResolving(
    "getBlocks",
    [],
    blockLibraryDataStore
  );
  const isFinishedLoadingBlocks = getHasFinishedResolution(
    "getBlocks",
    [],
    blockLibraryDataStore
  );

  // Query block keywords
  const blockKeywords = getBlockKeywords();
  const isLoadingKeywords = getIsResolving(
    "getBlockKeywords",
    [],
    blockLibraryDataStore
  );

  // Page state
  const [savedLibraryState, setSavedLibraryState] = useLocalStorage(
    `${cachedLibraryStateKey}-${contentType}`
  );
  const [libraryState, dispatch] = useReducer(reducer, initialState, () => {
    const libraryState = Object.assign(
      initialState,
      getBlockLibraryState(),
      savedLibraryState ?? {}
    );

    // Clear cache
    setSavedLibraryState(null);

    return libraryState;
  });

  const {
    sortType,
    selectedKeywords,
    isOpenHelp,
    currentPage = 1,
    pageSize = 6,
    importedItems,
  } = libraryState;

  const updateLibraryState = (payload) =>
    dispatchLibraryState(dispatch, payload);

  // Build data for grid
  const [fullBlocks, filteredBlocks] = useLibraryContent(
    libraryState,
    blockData
  );

  // Get grid loading status
  const isLoadingGrid =
    isLoadingBlocks || isLoadingPlugins || isUndefined(canManagePlugins);
  const isFinishedLoadingGrid =
    isFinishedLoadingBlocks && isFinishedLoadingPlugins;

  // Query available block names
  const availableBlockTypes = getBlockTypes();
  const availableBlockNames = map(availableBlockTypes, "name");

  const importItemHandler = ({
    item,
    isExisting,
    localBlocks,
    localVariations,
    setImportingMessages,
    setIsOpenConfirm,
  }) => {
    const existingItem = isExisting
      ? localBlocks.find(({ slug }) => slug === item?.slug)
      : false;

    // Import data
    importData({
      contentType,
      item,
      existingId: existingItem ? existingItem.id : false,
      localVariations,
      finishCallback: (messages) => {
        updateLibraryState({
          insertingItem: "",
          importedItems: [
            ...importedItems,
            ...messages.map(({ slug }) => slug),
          ],
        });
        setImportingMessages(messages);
        setIsOpenConfirm(false);
      },
    });

    // Submit download
    fetch(`${getBlockLibraryURL()}/wp-json/boldblocks/v1/submitDownloadBlock`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ id: item.id }),
      credentials: "omit",
    });
  };

  const {
    records: localBlocks,
    isResolving: isResolvingLocalBlocks,
  } = useEntityRecords("postType", "boldblocks_block", { per_page: 100 });

  const {
    records: localVariations,
    isResolving: isResolvingLocalVariations,
  } = useEntityRecords("postType", "boldblocks_variation", { per_page: 100 });

  const [importingMessages, setImportingMessages] = useState([]);

  // Memorized context value
  const contextValue = useMemo(() => {
    return {
      libraryState,
      updateLibraryState,
      availableBlockNames,
      plugins,
      canManagePlugins,
      onImportItem: importItemHandler,
      localBlocks,
      localVariations,
      isResolvingLocalData:
        isResolvingLocalBlocks || isResolvingLocalVariations,
      importingMessages,
      setImportingMessages,
      contentType,
    };
  }, [
    libraryState,
    updateLibraryState,
    availableBlockNames,
    plugins,
    canManagePlugins,
    importItemHandler,
    localBlocks,
    isResolvingLocalBlocks,
    localVariations,
    isResolvingLocalVariations,
    importingMessages,
    setImportingMessages,
    contentType,
  ]);

  return (
    <LibraryContext.Provider value={contextValue}>
      <PageStyled
        className={classnames("template-modal", className, {
          "is-locked":
            libraryState?.installingPlugins?.length ||
            libraryState?.activatingPlugins?.length ||
            libraryState?.insertingItem ||
            libraryState?.isReloading,
        })}
      >
        {importingMessages && (
          <div className="block-library-notices">
            {importingMessages.map(({ type, message, slug, id }) => (
              <Notice
                key={slug}
                className="block-library-notice"
                status={type}
                isDismissible={true}
                onRemove={() =>
                  setImportingMessages([
                    ...importingMessages.filter((item) => item.slug !== slug),
                  ])
                }
                actions={
                  id
                    ? [
                        {
                          label: __("Edit item", "content-blocks-builder"),
                          url: getEditPostURL(id),
                        },
                      ]
                    : []
                }
              >
                <span className="notice-message">{message}</span>
              </Notice>
            ))}
          </div>
        )}
        <div className="template-header">
          <div className="sort-box">
            <SortControl
              onChange={(value) => {
                updateLibraryState({
                  sortType: value,
                  currentPage: 1,
                });
              }}
              value={sortType}
            />
          </div>
          <KeywordsControl
            items={
              filteredBlocks.length !== blockData.length ? filteredBlocks : []
            }
            keywords={blockKeywords}
            isLoading={isLoadingKeywords}
            selectedKeywords={selectedKeywords}
            onChange={(newKeywords) => {
              updateLibraryState({
                selectedKeywords: newKeywords,
                currentPage: 1,
              });
            }}
          />
          <HeaderActionsControl
            isOpenHelp={isOpenHelp}
            onToggleHelp={() => {
              updateLibraryState({ isOpenHelp: !isOpenHelp });
            }}
          />
        </div>
        {isOpenHelp && (
          <Notice
            status="info"
            className="template-modal__legend"
            isDismissible={false}
          >
            <p>
              <strong>
                {__(
                  "Click on the preview to import.",
                  "content-blocks-builder"
                )}
              </strong>
            </p>
            <p>
              {__(
                "Items with Pro features like parallax, animations... require the Pro version to work full-functional. They will still work perfectly fine on the Free version but without Pro features.",
                "content-blocks-builder"
              )}
            </p>
            <p>
              {__(
                "Some items require blocks in external plugins. You have to install and activate those required plugins to use those items. All external plugins included in blocks are developed and maintained by us, so they are safe to use.",
                "content-blocks-builder"
              )}
            </p>
          </Notice>
        )}
        {isLoadingGrid && <Spinner />}
        <div className="template-list-wrapper scrollbar">
          {isFinishedLoadingGrid && (
            <>
              <TemplateGridControl
                items={fullBlocks}
                allItems={filteredBlocks}
                onChangePage={(i) => {
                  updateLibraryState({ currentPage: i });
                }}
                currentPage={currentPage}
                pageSize={pageSize}
                TemplateItemControl={TemplateItemControl}
              />
            </>
          )}
        </div>
      </PageStyled>
    </LibraryContext.Provider>
  );
};
/**
 * Kick start
 */
domReady(() => {
  render(
    <BlockLibraryPage />,
    document.querySelector(".js-boldblocks-settings-root")
  );
});
