/**
 * External dependencies
 */
import classnames from "classnames";
import { noop, isArray, find, isUndefined, isNil } from "lodash";

/**
 * WordPress dependencies
 */
import { __, sprintf } from "@wordpress/i18n";
import {
  Spinner,
  __experimentalConfirmDialog as ConfirmDialog,
} from "@wordpress/components";
import { BlockPreview } from "@wordpress/block-editor";
import { useState, useMemo, useContext } from "@wordpress/element";
import { createBlocksFromInnerBlocksTemplate } from "@wordpress/blocks";

/**
 * Internal dependencies
 */
import { log, useLocalStorage } from "sdk/utils";
import {
  decodeHtml,
  getDependentBlockTypes,
  getMissingBlocks,
  isPremium,
} from "../../utils";
import { SearchParams } from "../../utils/searchparams";
import {
  ItemLinks,
  ItemNotices,
  ItemProBadges,
  ItemDependentPlugins,
  DependentItems,
} from "../../components/template-modal/grid-control";
import { TemplateItemStyled } from "../../components/template-modal/styles";
import {
  LibraryContext,
  cachedLibraryStateKey,
  findExistVariation,
  getBlocksFromVariationData,
} from "../data";

const reloadPage = (
  libraryState,
  setSavedLibraryState,
  updateLibraryState,
  delay = 500
) => {
  updateLibraryState({ isReloading: true });
  setTimeout(() => {
    // Save current state
    setSavedLibraryState(libraryState);

    const searchParams = new SearchParams();
    searchParams.reload();
  }, delay);
};

const ConfirmModal = ({
  isOpen,
  setIsOpen,
  handleConfirm = noop,
  content = null,
}) => {
  return (
    <ConfirmDialog
      isOpen={isOpen}
      onConfirm={handleConfirm}
      onCancel={() => {
        setIsOpen(false);
      }}
    >
      {content}
    </ConfirmDialog>
  );
};

const BlockTitle = ({ title, item }) => {
  const parentTitle = item?.meta?.boldblocks_enable_repeater
    ? item?.meta?.boldblocks_parent_block_title ?? `${title} repeater`
    : "";
  return (
    <>
      <h3 className="template-item__title">
        {sprintf(__("Block name: %s", "content-blocks-builder"), title)}
      </h3>
      {parentTitle && (
        <h3 className="template-item__title">
          {sprintf(
            __("Parent block: %s", "content-blocks-builder"),
            parentTitle
          )}
        </h3>
      )}
    </>
  );
};

const VariationTitle = ({ title, item }) => {
  const blockName = item?.meta?.boldblocks_variation_block_name;
  return (
    <>
      <h3 className="template-item__title">
        {sprintf(__("Variation name: %s", "content-blocks-builder"), title)}
      </h3>
      {blockName && (
        <h3 className="template-item__title">
          {sprintf(__("Block name: %s", "content-blocks-builder"), blockName)}
        </h3>
      )}
    </>
  );
};

const useItemData = ({
  item,
  canManagePlugins,
  plugins,
  availableBlockNames,
  localVariations,
  isResolvingLocalData,
  isPremium,
  setItemNotices,
  contentType = "block",
}) => {
  return useMemo(() => {
    // The data has not fully loaded
    if (!item?.slug || isResolvingLocalData || isUndefined(canManagePlugins)) {
      return {};
    }

    let missingBlocks;
    let blocks;
    if (contentType === "block") {
      blocks = JSON.parse(item?.meta?.boldblocks_block_blocks ?? "[]");
    } else {
      blocks = getBlocksFromVariationData(
        item?.meta?.boldblocks_variation_data
      );
    }

    let dependentBlocks =
      item.meta[`boldblocks_${contentType}_dependent_blocks`] ?? [];

    if (!item?.thumbnail) {
      if ((!isArray(dependentBlocks) || !dependentBlocks.length) && blocks) {
        dependentBlocks = getDependentBlockTypes(blocks, []);
      }
    }

    if (isArray(dependentBlocks) && !!dependentBlocks.length) {
      missingBlocks = getMissingBlocks(dependentBlocks, availableBlockNames);
    }

    if (!missingBlocks?.length && blocks) {
      blocks = createBlocksFromInnerBlocksTemplate(blocks);
    }

    const tutorials = item?.meta?.boldblocks_tutorials;
    const externalResources = item?.meta?.boldblocks_external_resources;

    const dependentPlugins = !isNil(item?.meta?.boldblocks_dependencies)
      ? item.meta.boldblocks_dependencies
      : item?.meta?.boldblocks_block_dependencies;

    const missingPlugins =
      canManagePlugins && !!plugins.length && dependentPlugins
        ? dependentPlugins.filter(({ slug }) => {
            return !find(plugins, ["slug", slug]);
          })
        : [];

    const inactivePlugins =
      canManagePlugins && !!plugins.length && dependentPlugins
        ? dependentPlugins
            .map(({ slug }) => {
              const plugin = find(plugins, ["slug", slug]);
              if (plugin && plugin?.status === "inactive") {
                return plugin;
              } else {
                return false;
              }
            })
            .filter((i) => i)
        : [];

    const isPro = item?.meta?.boldblocks_is_pro;

    const hasProFeatures = item?.meta?.boldblocks_has_pro_features;

    const isExisting =
      contentType === "block"
        ? availableBlockNames.indexOf(`boldblocks/${item?.slug}`) > -1
        : item?.slug && localVariations
        ? findExistVariation(item?.slug, localVariations)
        : false;

    let notices = [];
    if (isExisting) {
      notices.push({
        type: "warning",
        message: __(
          "This item already exists. The import will override the existing item.",
          "content-blocks-builder"
        ),
      });
    }

    if (missingBlocks?.length) {
      notices.push({
        type: "warning",
        message: sprintf(
          __(
            "This item requires the following external block(s): %s. You must install and/or activate the required plugins to use it.",
            "content-blocks-builder"
          ),
          missingBlocks.join(", ")
        ),
      });

      if (!isUndefined(canManagePlugins) && !canManagePlugins) {
        notices.push({
          type: "warning",
          message: __(
            "You don't have permission to manage plugins, please contact the administrator for help.",
            "content-blocks-builder"
          ),
        });
      }
    } else {
      if (!!missingPlugins.length || !!inactivePlugins.length) {
        notices.push({
          type: "warning",
          message: __(
            "You must install and/or activate the required plugin(s) to use this item.",
            "content-blocks-builder"
          ),
        });
      }
    }

    if (!isPremium && isPro) {
      notices.push({
        type: "warning",
        message: __(
          "This item requires the Pro version of the plugin.",
          "content-blocks-builder"
        ),
      });
    }

    setItemNotices([...notices]);

    return {
      blocks,
      missingBlocks,
      tutorials,
      externalResources,
      dependentPlugins,
      missingPlugins,
      inactivePlugins,
      isPro,
      hasProFeatures,
      isExisting,
    };
  }, [
    item?.slug,
    canManagePlugins,
    plugins,
    isPremium,
    localVariations,
    isResolvingLocalData,
  ]);
};

export const TemplateItemControl = ({ item }) => {
  const {
    libraryState,
    updateLibraryState,
    onImportItem,
    availableBlockNames,
    plugins,
    canManagePlugins,
    localBlocks,
    localVariations,
    isResolvingLocalData,
    importingMessages,
    setImportingMessages,
    contentType,
  } = useContext(LibraryContext);

  const [, setSavedLibraryState] = useLocalStorage(
    `${cachedLibraryStateKey}-${contentType}`
  );

  const reload = () =>
    reloadPage(libraryState, setSavedLibraryState, updateLibraryState);

  const [itemNotices, setItemNotices] = useState([]);
  const {
    blocks,
    missingBlocks,
    tutorials,
    externalResources,
    dependentPlugins = [],
    missingPlugins,
    inactivePlugins,
    isPro,
    hasProFeatures,
    isExisting,
  } = useItemData({
    contentType,
    item,
    canManagePlugins,
    plugins,
    availableBlockNames,
    localVariations,
    isResolvingLocalData,
    setItemNotices,
  });

  const canInsert =
    !missingBlocks?.length &&
    !missingPlugins?.length &&
    !inactivePlugins?.length &&
    (!isPro || (isPro && isPremium));

  const onPluginInstalled = (name) => {
    setItemNotices([
      ...itemNotices,
      {
        type: "success",
        message: sprintf(
          __(
            "The plugin %s has been installed and activated.",
            "content-blocks-builder"
          ),
          name
        ),
      },
      {
        type: "success",
        message: __("Reloading the page.", "content-blocks-builder"),
      },
    ]);

    // Reload the library
    reload();
  };

  const onPluginActivated = (name) => {
    setItemNotices([
      ...itemNotices,
      {
        type: "success",
        message: sprintf(
          __("The plugin %s has been activated.", "content-blocks-builder"),
          name
        ),
      },
      {
        type: "success",
        message: __("Reloading the page.", "content-blocks-builder"),
      },
    ]);

    // Reload the library
    reload();
  };

  const [isOpenConfirm, setIsOpenConfirm] = useState(false);

  const itemPreview = useMemo(() => {
    if (!item?.slug) {
      return <div className="template-item__preview"></div>;
    }

    return (
      <div className="template-item__preview">
        {!!item?.thumbnail ? (
          <div
            className="template-item__thumbnail scrollbar"
            dangerouslySetInnerHTML={{ __html: item?.thumbnail }}
          />
        ) : (
          !missingBlocks?.length && (
            <BlockPreview blocks={blocks} viewportWidth={1400} />
          )
        )}
        <ItemProBadges isPro={isPro} hasProFeatures={hasProFeatures} />
        <ItemNotices notices={itemNotices} />
      </div>
    );
  }, [item?.slug, missingBlocks, libraryState?.insertingItem, itemNotices]);

  const title = decodeHtml(item.title);

  const description =
    contentType === "block"
      ? item?.meta?.boldblocks_enable_repeater
        ? item?.meta?.boldblocks_parent_block_description
        : item?.meta?.boldblocks_block_description
      : item?.meta?.boldblocks_variation_description;

  const handleImporting = () => {
    updateLibraryState({ insertingItem: item.id });
    setTimeout(() => {
      onImportItem({
        contentType,
        item,
        isExisting,
        localBlocks,
        localVariations,
        importingMessages,
        setImportingMessages,
        setIsOpenConfirm,
      });
    }, 0);
  };

  const isImported =
    libraryState?.importedItems?.length &&
    libraryState.importedItems.indexOf(item?.slug) > -1;

  return (
    <>
      <TemplateItemStyled
        className={classnames("template-item", {
          "is-ready": canInsert && !isImported && !isResolvingLocalData,
          "is-pro": isPro,
          "has-pro-features": hasProFeatures,
          "has-missing-blocks": missingBlocks?.length,
          "require-pro": !isPremium && isPro,
          "is-loading-data": !!item?.loadingFullData || isResolvingLocalData,
          "is-inserting": libraryState?.insertingItem === item.id,
          "is-existing": isExisting,
        })}
      >
        {(item?.loadingFullData ||
          libraryState?.insertingItem === item.id ||
          isResolvingLocalData) && <Spinner />}
        <div
          className="template-item__preview-wrapper"
          onClick={
            !canInsert ||
            libraryState?.insertingItem ||
            isImported ||
            isResolvingLocalData
              ? noop
              : () => {
                  if (isExisting) {
                    setIsOpenConfirm(true);
                  } else {
                    handleImporting();
                  }
                }
          }
        >
          {itemPreview}
        </div>
        <div className="template-item__footer scrollbar">
          <div className="template-item__title-details">
            {contentType === "block" ? (
              <BlockTitle title={title} item={item} />
            ) : (
              <VariationTitle title={title} item={item} />
            )}
          </div>

          {description && (
            <div className="template-item__description">{description}</div>
          )}

          <div className="template-item__actions">
            {contentType === "block" && (
              <DependentItems
                items={item?.variations ?? []}
                label={__("Variations:", "content-blocks-builder")}
                type="variation"
              />
            )}
            <ItemLinks
              links={tutorials}
              label={__("Tutorials:", "content-blocks-builder")}
            />
            <ItemLinks
              links={externalResources}
              label={__("Resources:", "content-blocks-builder")}
            />
            <ItemLinks
              links={dependentPlugins.map(({ slug, name: title }) => ({
                url: `https://wordpress.org/plugins/${slug}`,
                title,
              }))}
              label={__("Dependencies:", "content-blocks-builder")}
            />
          </div>

          <ItemDependentPlugins
            {...{
              missingPlugins,
              inactivePlugins,
              libraryState,
              updateLibraryState,
              onPluginInstalled,
              onPluginActivated,
            }}
          />
        </div>
      </TemplateItemStyled>
      {isExisting && (
        <ConfirmModal
          isOpen={isOpenConfirm}
          setIsOpen={setIsOpenConfirm}
          handleConfirm={() => {
            handleImporting();
            setIsOpenConfirm(false);
          }}
          content={
            <>
              {sprintf(
                __("Are you sure to import this %s?", "content-blocks-builder"),
                contentType
              )}
              <br />
              <strong>
                {sprintf(
                  __(
                    "This will override the existing %s!",
                    "content-blocks-builder"
                  ),
                  contentType
                )}
              </strong>
            </>
          }
        />
      )}
    </>
  );
};
