/**
 * External dependencies
 */
import { noop } from "lodash";

/**
 * WordPress dependencies
 */
import { sprintf, __ } from "@wordpress/i18n";
import { createContext, useMemo, useReducer } from "@wordpress/element";
import { useDispatch } from "@wordpress/data";
import apiFetch from "@wordpress/api-fetch";

/**
 * Internal dependencies
 */
import { store as blockLibrarysDataStore } from "./store";
import {
  filterItemsByKeywords,
  sortItemCb,
} from "../components/template-modal/data";

// Modal context
export const LibraryContext = createContext();

export const cachedLibraryStateKey = "bb-library-state";

// Initial state
export const initialState = {
  pageSize: 6,
  currentPage: 1,
  sortType: "featured",
  selectedKeywords: [],
  isOpenHelp: false,
  insertingItem: "",
  importedItems: [],
  isReloading: false,
  installingPlugins: [],
  activatingPlugins: [],
  contentType: "block",
};

export const reducer = (state, action) => {
  switch (action.type) {
    case "UPDATE_STATE": {
      return { ...state, ...action.payload };
    }

    default:
      return state;
  }
};

export const dispatchLibraryState = (dispatch, payload) =>
  dispatch({
    type: "UPDATE_STATE",
    payload,
  });

export const useLibraryContent = (libraryState, items) => {
  const {
    pageSize,
    currentPage,
    sortType,
    selectedKeywords,
    contentType = "block",
  } = libraryState;

  const { loadFullBlocks, loadFullVariations } = useDispatch(
    blockLibrarysDataStore
  );

  const loadFullItems =
    contentType === "block" ? loadFullBlocks : loadFullVariations;

  // Filter by keywords
  const keywordIds = selectedKeywords.join(",");
  const filteredItems = useMemo(() => {
    return filterItemsByKeywords(selectedKeywords, items);
  }, [keywordIds, items]);

  // Sorting
  const sortedItems = useMemo(() => {
    return filteredItems.sort(
      sortItemCb(sortType, {
        "30_days": "boldblocks_download_30_count",
        "7_days": "boldblocks_download_7_count",
      })
    );
  }, [filteredItems, sortType]);

  let currentPageItems = sortedItems.slice(
    (currentPage - 1) * pageSize,
    pageSize * currentPage
  );

  currentPageItems = currentPageItems.map((item) => {
    return { ...item, loadingFullData: !item?.slug };
  });

  let itemIds = currentPageItems
    .map(({ id, loadingFullData }) => (loadingFullData ? id : false))
    .filter((i) => i);

  const [, forceRender] = useReducer(() => ({}));
  if (itemIds.length) {
    const result = loadFullItems(itemIds);
    result.then(() => {
      forceRender();
    });
  }

  return [currentPageItems, sortedItems];
};

export const getBlockLibraryURL = (contentType = "block") => {
  let url =
    contentType === "block"
      ? window?.BoldBlocksBlockLibrary?.URL
      : window?.BoldBlocksVariationLibrary?.URL;

  if (!url) {
    url = "https://boldpatterns.net";
  }

  return url;
};

export const findExistVariation = (slug, localVariations) =>
  localVariations.find((item) => slug === item.slug);

const buildBlocksFromInnerTemplate = (blocks) => {
  let blockData = [];
  if (!blocks.length) {
    return blockData;
  }

  for (let i = 0; i < blocks.length; i++) {
    let [name, attributes, innerBlocks] = blocks[i];
    innerBlocks = buildBlocksFromInnerTemplate(innerBlocks);

    blockData.push({ name, attributes, innerBlocks });
  }

  return blockData;
};

export const getBlocksFromVariationData = (variationDataRaw) => {
  if (!variationDataRaw) {
    return [];
  }

  const variationData = JSON.parse(variationDataRaw);

  const {
    blockName: name,
    variation: { attributes, innerBlocks },
  } = variationData;

  return [
    {
      name,
      attributes,
      innerBlocks: buildBlocksFromInnerTemplate(innerBlocks),
    },
  ];
};

export const importData = ({
  contentType,
  item,
  existingId = false,
  localVariations,
  finishCallback,
}) => {
  if (contentType === "block") {
    importBlock({ item, existingId, localVariations, finishCallback });
  } else {
    importVariation({ item, existingId, finishCallback });
  }
};

export const importBlock = async ({
  item,
  existingId = false,
  localVariations,
  finishCallback = noop,
}) => {
  const {
    title,
    slug,
    content,
    keywords,
    variations,
    parentVariations,
    meta: {
      boldblocks_is_pro,
      boldblocks_has_pro_features,
      boldblocks_download_count,
      boldblocks_download_7_count,
      boldblocks_download_30_count,
      boldblocks_download_stats,
      boldblocks_tutorials,
      boldblocks_external_resources,
      ...meta
    } = {},
  } = item;

  const block = {
    title,
    slug,
    content,
    meta,
    keywords,
  };

  const variationData = [];

  if (variations && variations?.length) {
    variationData.push(
      ...variations.map(({ title, content, slug, meta }) => ({
        title,
        content,
        slug,
        meta,
      }))
    );
  }

  if (parentVariations && parentVariations?.length) {
    variationData.push(
      ...parentVariations.map(({ title, content, slug, meta }) => ({
        title,
        content,
        slug,
        meta,
      }))
    );
  }

  let messages = [];

  return apiFetch({
    path: existingId
      ? `wp/v2/boldblocks-blocks/${existingId}`
      : "wp/v2/boldblocks-blocks",
    method: "POST",
    data: { ...block, status: "publish" },
  })
    .then((res) => {
      messages.push({
        id: res.id,
        slug: block.slug,
        type: "success",
        message: sprintf(
          __(
            "The block '%s' has been imported successfully.",
            "content-blocks-builder"
          ),
          block.title
        ),
      });
      if (variationData.length) {
        Promise.all(
          variationData.map(async (item) => {
            const existVariation = findExistVariation(
              item.slug,
              localVariations
            );
            return apiFetch({
              path: !existVariation
                ? "boldblocks/v1/createVariation"
                : `wp/v2/boldblocks-variations/${existVariation.id}`,
              method: "POST",
              data: { ...item, status: "publish" },
            })
              .then((res) => ({
                id: existVariation ? res.id : res?.post?.id,
                slug: item.slug,
                type: "success",
                message: sprintf(
                  __(
                    "The variation '%s' has been imported successfully.",
                    "content-blocks-builder"
                  ),
                  item.title
                ),
              }))
              .catch((error) => {
                console.error(error);
                return {
                  slug: item.slug,
                  type: "error",
                  message: sprintf(
                    __(
                      "Failed to import variation: '%s'.",
                      "content-blocks-builder"
                    ),
                    item.title
                  ),
                };
              });
          })
        ).then((res) => {
          messages.push(...res);

          finishCallback(messages);
        });
      } else {
        finishCallback(messages);
      }

      return messages;
    })
    .catch((error) => {
      console.error(error);

      messages.push({
        slug: block.slug,
        type: "error",
        message: sprintf(
          __("Failed to import block: '%s'.", "content-blocks-builder"),
          block.title
        ),
      });

      finishCallback(messages);

      return messages;
    });
};

export const importVariation = async ({
  item,
  existingId = false,
  finishCallback = noop,
}) => {
  const {
    title,
    slug,
    content,
    meta: {
      boldblocks_is_pro,
      boldblocks_has_pro_features,
      boldblocks_download_count,
      boldblocks_download_7_count,
      boldblocks_download_30_count,
      boldblocks_download_stats,
      boldblocks_tutorials,
      boldblocks_external_resources,
      boldblocks_is_queryable,
      ...meta
    } = {},
  } = item;

  const variation = {
    title,
    slug,
    content,
    meta,
  };

  let messages = [];

  return apiFetch({
    path: existingId
      ? `wp/v2/boldblocks-variations/${existingId}`
      : "boldblocks/v1/createVariation",
    method: "POST",
    data: { ...variation, status: "publish" },
  })
    .then((res) => {
      messages.push({
        id: existingId ? res.id : res?.post?.id,
        slug: variation.slug,
        type: "success",
        message: sprintf(
          __(
            "The variation '%s' has been imported successfully.",
            "content-blocks-builder"
          ),
          variation.title
        ),
      });

      finishCallback(messages);

      return messages;
    })
    .catch((error) => {
      console.error(error);

      messages.push({
        slug: variation.slug,
        type: "error",
        message: sprintf(
          __("Failed to import variation: '%s'.", "content-blocks-builder"),
          variation.title
        ),
      });

      finishCallback(messages);

      return messages;
    });
};
