/**
 * External dependencies
 */
import { isObject, isEmpty } from "lodash";
import fuzzysort from "fuzzysort";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { createContext, useMemo, useReducer } from "@wordpress/element";
import { useDispatch } from "@wordpress/data";

/**
 * Internal dependencies
 */
import { useDebounceValue } from "../../utils";
import { store as boldblocksDataStore } from "../../store";

// Modal context
export const ModalContext = createContext();

export const cachedModalStateKey = "bb-patterns-modal-state";

// Initial state
export const initialState = {
  pageSize: 6,
  currentPage: 1,
  sortType: "featured",
  selectedKeywords: [],
  isOpenHelp: false,
  searchTerm: "",
  insertingPattern: "",
  isReloading: false,
  installingPlugins: [],
  activatingPlugins: [],
};

export const reducer = (state, action) => {
  switch (action.type) {
    case "UPDATE_STATE": {
      return { ...state, ...action.payload };
    }

    default:
      return state;
  }
};

export const dispatchModalState = (dispatch, payload) =>
  dispatch({
    type: "UPDATE_STATE",
    payload,
  });

export const refinePreviewData = (rawBlocks) => {
  rawBlocks = (rawBlocks ?? "").replaceAll(
    /\w+:\/\/\S*(w=(\d*))&(h=(\d*))&\w+\S*"/g,
    (url, w, width, h, height) => {
      if (width > 800) {
        return url
          .replace(w, "w=" + Math.floor(Number(width) / 2))
          .replace(h, "h=" + Math.floor(Number(height) / 2));
      }

      return url;
    }
  );

  let blocks;
  if (rawBlocks) {
    blocks = JSON.parse(rawBlocks);

    if (!!blocks.length) {
      const { attributes: { boldblocks: { height } = {} } = {} } =
        blocks[0] ?? {};

      if (isObject(height) && !isEmpty(height)) {
        let { lg: { value: { value } } = {} } = height;

        if (value === "100vh") {
          blocks[0] = {
            ...blocks[0],
            attributes: {
              ...blocks[0].attributes,
              boldblocks: {
                ...blocks[0].attributes.boldblocks,
                height: {
                  ...height,
                  lg: { value: { height: "96vh", value: "96vh" } },
                },
              },
            },
          };
        }
      }
    }
  }
  return blocks;
};

export const filterItemsByKeywords = (keywords, items) => {
  let filteredItems = [];
  if (!keywords.length || !items.length) {
    return items;
  }

  const [firstKeyword, ...restKeywords] = keywords;
  filteredItems = items.filter(
    ({ keywordIds: keywords }) => keywords.indexOf(firstKeyword.id) > -1
  );

  return filterItemsByKeywords(restKeywords, filteredItems);
};

const searchOptions = {
  limit: 60,
  threshold: -100,
  keys: ["title", "keywords", "description"],
};
const searchPatterns = (searchTerm, patterns) => {
  let searchPatterns;
  if (!searchTerm || searchTerm.length < 3) {
    searchPatterns = patterns;
  } else {
    const results = fuzzysort.go(searchTerm, patterns, searchOptions);
    if (results.length) {
      searchPatterns = results.map(({ obj }) => obj);
    } else {
      searchPatterns = [];
    }
  }

  return searchPatterns;
};

export const sortItemCb = (
  sortType,
  args = { "30_days": "count_30", "7_days": "count_7" }
) => {
  let cb;

  if (sortType === "featured") {
    cb = (a, b) => b.order - a.order;
  } else if (sortType === "latest") {
    cb = (a, b) => b.id - a.id;
  } else if (sortType === "30_days") {
    cb = (a, b) => b.meta[args["30_days"]] - a.meta[args["30_days"]];
  } else if (sortType === "7_days") {
    cb = (a, b) => b.meta[args["7_days"]] - a.meta[args["7_days"]];
  }
  return cb;
};

export const useModalContent = (modalState, patterns) => {
  const {
    pageSize,
    currentPage,
    sortType,
    selectedKeywords,
    searchText,
  } = modalState;

  const { loadFullPatterns } = useDispatch(boldblocksDataStore);

  // Filter by keywords
  const keywordIds = selectedKeywords.join(",");
  const filteredPatterns = useMemo(() => {
    return filterItemsByKeywords(selectedKeywords, patterns);
  }, [keywordIds, patterns]);

  // Search
  const [searchTerm] = useDebounceValue(searchText);
  const searchedPatterns = useMemo(() => {
    if (searchTerm && searchTerm.length >= 3) {
      return searchPatterns(searchTerm, filteredPatterns);
    }

    return filteredPatterns;
  }, [searchTerm, filteredPatterns]);

  // Sorting
  const sortedPatterns = useMemo(() => {
    if (!(searchTerm && searchTerm.length >= 3)) {
      return searchedPatterns.sort(sortItemCb(sortType));
    }

    return searchedPatterns;
  }, [searchedPatterns, sortType, searchTerm]);

  let currentPagePatterns = sortedPatterns.slice(
    (currentPage - 1) * pageSize,
    pageSize * currentPage
  );

  currentPagePatterns = currentPagePatterns.map((pattern) => {
    return { ...pattern, loadingFullData: !pattern?.slug };
  });

  let patternIds = currentPagePatterns
    .map(({ id, loadingFullData }) => (loadingFullData ? id : false))
    .filter((i) => i);

  const [, forceRender] = useReducer(() => ({}));
  if (patternIds.length) {
    const result = loadFullPatterns(patternIds);
    result.then(() => {
      forceRender();
    });
  }

  return [currentPagePatterns, sortedPatterns];
};

export const getPatternLibraryURL = () => {
  return window?.BoldBlocksPatternLibrary?.URL
    ? BoldBlocksPatternLibrary.URL
    : "https://boldpatterns.net";
};
