/**
 * External dependencies
 */
import classnames from "classnames";
import { noop, isArray, find, isUndefined, map } from "lodash";

/**
 * WordPress dependencies
 */
import { __, sprintf } from "@wordpress/i18n";
import { Spinner, Button, Notice, ExternalLink } from "@wordpress/components";
import { BlockPreview } from "@wordpress/block-editor";
import { useState, useMemo, useContext } from "@wordpress/element";
import { createBlocksFromInnerBlocksTemplate } from "@wordpress/blocks";
import apiFetch from "@wordpress/api-fetch";
import { addQueryArgs } from "@wordpress/url";

/**
 * Internal dependencies
 */
import { PaginateLinks } from "sdk/components";
import { log, useLocalStorage } from "sdk/utils";
import {
  decodeHtml,
  getDependentBlockTypes,
  getMissingBlocks,
  isPremium,
} from "../../utils";
import { SearchParams } from "../../utils/searchparams";
import { TemplateGridStyled, TemplateItemStyled } from "./styles";
import {
  ModalContext,
  refinePreviewData,
  cachedModalStateKey,
  getPatternLibraryURL,
} from "./data";
import { importBlock, importVariation } from "../../block-library/data";

export const ItemProBadges = ({ isPro, hasProFeatures }) => {
  return (
    <>
      {(isPro || hasProFeatures) && (
        <div className="template-item__badges">
          {isPro && (
            <span className="pro-item">
              {__("Pro", "content-blocks-builder")}
            </span>
          )}
          {hasProFeatures && (
            <span className="pro-features-item">
              {__("Has pro features", "content-blocks-builder")}
            </span>
          )}
        </div>
      )}
    </>
  );
};

export const ItemLinks = ({ links, label }) => {
  return (
    <>
      {!!links?.length && (
        <ul className="template-item__links">
          <li className="template-item__label">
            <strong>{label}</strong>
          </li>
          {links.map(({ url, title }, index) => (
            <li className="template-item__link" key={index}>
              <a href={url} target="_blank">
                {title}
              </a>
            </li>
          ))}
        </ul>
      )}
    </>
  );
};

export const ItemNotices = ({ notices }) => {
  return (
    <>
      {!!notices?.length && (
        <div className="item-notices">
          {notices.map(
            ({ type, message, actions = [], customActions = null }, index) => (
              <Notice
                className="item-notice"
                status={type}
                isDismissible={false}
                key={`notice-${index}`}
                actions={actions}
              >
                {message}
                {customActions}
              </Notice>
            )
          )}
        </div>
      )}
    </>
  );
};

export const DependentItems = ({ items, label, type }) => {
  return (
    <>
      {!!items?.length && (
        <ul className="template-item__links">
          {!!label && (
            <li className="template-item__label">
              <strong>{label}</strong>
            </li>
          )}
          {items.map(({ title }, index) => (
            <li className="template-item__link" key={`${type}-${index}`}>
              <code>{title}</code>
            </li>
          ))}
        </ul>
      )}
    </>
  );
};

export const ItemDependentPlugins = ({
  missingPlugins,
  inactivePlugins,
  libraryState,
  updateLibraryState,
  onPluginInstalled,
  onPluginActivated,
}) => {
  const { activatingPlugins = [], installingPlugins = [] } = libraryState;

  const setActivatingPlugins = (plugins) =>
    updateLibraryState({ activatingPlugins: plugins });
  const setInstallingPlugins = (plugins) =>
    updateLibraryState({ installingPlugins: plugins });

  return (
    <>
      {(!!missingPlugins?.length || !!inactivePlugins?.length) && (
        <div className="template-item__actions">
          {!!missingPlugins.length &&
            missingPlugins.map(
              ({ name, slug }) =>
                !libraryState?.isReloading && (
                  <Button
                    key={slug}
                    variant="primary"
                    size="small"
                    disabled={!!installingPlugins.length}
                    onClick={(e) => {
                      e.preventDefault();
                      setInstallingPlugins([...installingPlugins, slug]);
                      apiFetch({
                        path: `wp/v2/plugins`,
                        method: "POST",
                        data: { slug, status: "active" },
                      })
                        .then(() => {
                          onPluginInstalled(name);
                        })
                        .catch((error) => log(error, "error"))
                        .finally(() =>
                          setInstallingPlugins(
                            installingPlugins.filter((i) => i !== slug)
                          )
                        );
                    }}
                  >
                    {sprintf(__("Install %s", "content-blocks-builder"), name)}
                    {!!installingPlugins.length &&
                      installingPlugins.indexOf(slug) !== -1 && (
                        <Spinner style={{ margin: "0 5px" }} />
                      )}
                  </Button>
                )
            )}
          {!!inactivePlugins.length &&
            inactivePlugins.map(
              ({ name, plugin, slug }) =>
                !libraryState?.isReloading && (
                  <Button
                    key={slug}
                    variant="primary"
                    size="small"
                    disabled={!!activatingPlugins.length}
                    onClick={(e) => {
                      e.preventDefault();
                      setActivatingPlugins([...activatingPlugins, slug]);
                      apiFetch({
                        path: `wp/v2/plugins/${plugin}`,
                        method: "POST",
                        data: { status: "active" },
                      })
                        .then(() => {
                          onPluginActivated(name);
                        })
                        .catch((error) => log(error, "error"))
                        .finally(() => {
                          setActivatingPlugins(
                            activatingPlugins.filter((i) => i !== slug)
                          );
                        });
                    }}
                  >
                    {sprintf(__("Activate %s", "content-blocks-builder"), name)}
                    {!!activatingPlugins.length &&
                      activatingPlugins.indexOf(slug) !== -1 && (
                        <Spinner style={{ margin: "0 5px" }} />
                      )}
                  </Button>
                )
            )}
        </div>
      )}
    </>
  );
};

const ImportActions = ({ label, handleClick }) => {
  const [isImporting, setIsImporting] = useState(false);
  const [isImported, setIsImported] = useState(false);

  return (
    <div className="components-notice__actions">
      <Button
        variant="primary"
        size="small"
        className="components-notice__action"
        disabled={isImporting || isImported}
        onClick={(e) => {
          e.preventDefault();

          setIsImporting(true);
          handleClick({ setIsImported, setIsImporting });
        }}
      >
        {label}
        {isImporting && <Spinner style={{ margin: "0 5px" }} />}
      </Button>
    </div>
  );
};

const useItemData = ({
  item,
  canManagePlugins,
  plugins,
  availableBlockNames,
  localBlocks,
  localVariations,
  isPremium,
  setItemNotices,
  isItemLoadingData,
  reload,
}) => {
  return useMemo(() => {
    // The data has not fully loaded
    if (isItemLoadingData || !item?.slug || isUndefined(canManagePlugins)) {
      return {};
    }

    let missingCustomVariations = [];

    const customVariations = item?.variations ?? [];
    if (customVariations?.length && localVariations?.length) {
      const localVariationSlugs = map(localVariations, "slug");
      missingCustomVariations = customVariations.filter(
        ({ slug }) => localVariationSlugs.indexOf(slug) === -1
      );
    }

    const customBlocks = item?.libraryBlocks ?? [];
    const customBlockTypes = customBlocks.map(
      ({ slug }) => `boldblocks/${slug}`
    );

    let missingCustomBlocks = [];
    if (customBlocks?.length) {
      const localBlockSlugs = map(localBlocks, "slug");
      missingCustomBlocks = customBlocks.filter(
        ({ slug }) => localBlockSlugs.indexOf(slug) === -1
      );
    }

    let missingBlocks;
    let blocks = refinePreviewData(item.meta?.blocks);
    let dependentBlocks = item.meta?.dependent_blocks;

    if (!item?.thumbnail) {
      if ((!isArray(dependentBlocks) || !dependentBlocks.length) && blocks) {
        dependentBlocks = getDependentBlockTypes(blocks, []);
      }
    }

    if (isArray(dependentBlocks) && !!dependentBlocks.length) {
      missingBlocks = getMissingBlocks(dependentBlocks, availableBlockNames);
    }

    if (!missingBlocks?.length && blocks) {
      blocks = createBlocksFromInnerBlocksTemplate(blocks);
    }

    const tutorials = item?.meta?.tutorials;
    const externalResources = item?.meta?.external_resources;

    const dependentPlugins = item.meta?.dependencies;
    const missingPlugins =
      canManagePlugins && !!plugins.length && dependentPlugins
        ? dependentPlugins.filter(({ slug }) => {
            return !find(plugins, ["slug", slug]);
          })
        : [];

    const inactivePlugins =
      canManagePlugins && !!plugins.length && dependentPlugins
        ? dependentPlugins
            .map(({ slug }) => {
              const plugin = find(plugins, ["slug", slug]);
              if (plugin && plugin?.status === "inactive") {
                return plugin;
              } else {
                return false;
              }
            })
            .filter((i) => i)
        : [];

    const isPro = item?.meta?.is_pro;

    const hasProFeatures = item?.meta?.has_pro_features;

    let notices = [];

    if (missingBlocks?.length) {
      const missingExternalBlocks = missingBlocks.filter(
        (blockName) => customBlockTypes.indexOf(blockName) === -1
      );

      if (missingExternalBlocks?.length) {
        notices.push({
          type: "warning",
          message: sprintf(
            __(
              "This item requires the following external block(s): %s. You must install and/or activate the required plugins to use it.",
              "content-blocks-builder"
            ),
            missingExternalBlocks.join(", ")
          ),
        });
      }

      if (missingCustomBlocks?.length || missingCustomVariations?.length) {
        notices.push({
          type: "warning",
          message: (
            <>
              {__(
                "This item requires some custom blocks and/or custom variations from the library.",
                "content-blocks-builder"
              )}
              {canManagePlugins && (
                <>
                  &nbsp;
                  <ExternalLink
                    href={addQueryArgs(
                      "edit.php?post_type=boldblocks_block&page=cbb-block-library"
                    )}
                  >
                    {__("Browse block library", "content-blocks-builder")}
                  </ExternalLink>
                  &nbsp;
                  <ExternalLink
                    href={addQueryArgs(
                      "edit.php?post_type=boldblocks_block&page=cbb-variation-library"
                    )}
                  >
                    {__("Browse variation library", "content-blocks-builder")}
                  </ExternalLink>
                </>
              )}
            </>
          ),
          customActions: (
            <>
              {canManagePlugins && (
                <ImportActions
                  label={__(
                    "Import all required items from library",
                    "content-blocks-builder"
                  )}
                  handleClick={({ setIsImported, setIsImporting }) => {
                    Promise.all([
                      Promise.all(
                        missingCustomBlocks.map(async (item) =>
                          importBlock({ item })
                        )
                      ).then((blocksRes) => {
                        const importedNotices = [];

                        blocksRes.forEach((messages) =>
                          messages.forEach((message) =>
                            importedNotices.push(message)
                          )
                        );

                        return importedNotices;
                      }),
                      Promise.all(
                        missingCustomVariations.map(async (item) =>
                          importVariation({ item })
                        )
                      ).then((variationsRes) => {
                        const importedNotices = [];

                        variationsRes.forEach((messages) =>
                          messages.forEach((message) =>
                            importedNotices.push(message)
                          )
                        );

                        return importedNotices;
                      }),
                    ]).then((combineRes) => {
                      setIsImporting(false);
                      setIsImported(true);

                      let importedNotices = [];
                      combineRes.forEach((messagess) => {
                        importedNotices = [...importedNotices, ...messagess];
                      });

                      importedNotices = [
                        ...importedNotices,
                        {
                          type: "success",
                          message: __(
                            "Reloading the page.",
                            "content-blocks-builder"
                          ),
                        },
                      ];

                      // Update notices
                      setItemNotices([...notices, ...importedNotices]);

                      // Reload the modal
                      reload();
                    });
                  }}
                />
              )}
            </>
          ),
        });
      }

      if (!isUndefined(canManagePlugins) && !canManagePlugins) {
        notices.push({
          type: "warning",
          message: __(
            "You don't have permission to manage plugins, please contact the administrator for help.",
            "content-blocks-builder"
          ),
        });
      }
    } else {
      if (!!missingPlugins.length || !!inactivePlugins.length) {
        notices.push({
          type: "warning",
          message: __(
            "You must install and/or activate the required plugin(s) to use this item.",
            "content-blocks-builder"
          ),
        });
      }
    }

    if (!isPremium && isPro) {
      notices.push({
        type: "warning",
        message: __(
          "This item requires the Pro version of the plugin.",
          "content-blocks-builder"
        ),
      });
    }

    setItemNotices([...notices]);

    return {
      blocks,
      missingBlocks,
      tutorials,
      externalResources,
      dependentPlugins,
      customVariations,
      customBlocks,
      missingPlugins,
      inactivePlugins,
      isPro,
      hasProFeatures,
    };
  }, [item?.slug, canManagePlugins, plugins, isPremium, isItemLoadingData]);
};

export const TemplateItemControl = ({ item: pattern }) => {
  const {
    modalState,
    updateModalState,
    onInsertPattern,
    availableBlockNames,
    plugins,
    canManagePlugins,
    localBlocks,
    isResolvingLocalBlocks,
    localVariations,
    isResolvingLocalVariations,
  } = useContext(ModalContext);

  const [, setSavedModalState] = useLocalStorage(cachedModalStateKey);

  const reload = () =>
    reloadModal(modalState, setSavedModalState, updateModalState);

  const title = decodeHtml(pattern.title);

  const isItemLoadingData =
    pattern?.loadingFullData ||
    isResolvingLocalBlocks ||
    isResolvingLocalVariations;

  const [itemNotices, setItemNotices] = useState([]);
  const {
    blocks,
    missingBlocks,
    tutorials,
    externalResources,
    dependentPlugins = [],
    customVariations,
    customBlocks,
    missingPlugins,
    inactivePlugins,
    isPro,
    hasProFeatures,
  } = useItemData({
    item: pattern,
    canManagePlugins,
    plugins,
    availableBlockNames,
    localBlocks,
    localVariations,
    setItemNotices,
    isItemLoadingData,
    reload,
  });

  const canInsert =
    !isItemLoadingData &&
    !missingBlocks?.length &&
    !missingPlugins?.length &&
    !inactivePlugins?.length &&
    (!isPro || (isPro && isPremium));

  const itemPreview = useMemo(() => {
    if (!pattern?.slug) {
      return <div className="template-item__preview"></div>;
    }

    return (
      <div
        className="template-item__preview"
        onClick={
          !canInsert || modalState?.insertingPattern
            ? noop
            : () => {
                updateModalState({ insertingPattern: pattern.id });
                setTimeout(() => {
                  onInsertPattern({ id: pattern.id, title }, blocks);
                }, 0);
              }
        }
      >
        {!!pattern?.thumbnail ? (
          <div
            className="template-item__thumbnail scrollbar"
            dangerouslySetInnerHTML={{ __html: pattern?.thumbnail }}
          />
        ) : (
          !missingBlocks?.length && (
            <BlockPreview blocks={blocks} viewportWidth={1400} />
          )
        )}

        <ItemProBadges isPro={isPro} hasProFeatures={hasProFeatures} />
        <ItemNotices notices={itemNotices} />
      </div>
    );
  }, [pattern?.slug, modalState?.insertingPattern, itemNotices]);

  const onPluginInstalled = (name) => {
    setItemNotices([
      ...itemNotices,
      {
        type: "success",
        message: sprintf(
          __(
            "The plugin %s has been installed and activated.",
            "content-blocks-builder"
          ),
          name
        ),
      },
      {
        type: "success",
        message: __("Reloading the page.", "content-blocks-builder"),
      },
    ]);

    // Reload the modal
    reload();
  };

  const onPluginActivated = (name) => {
    setItemNotices([
      ...itemNotices,
      {
        type: "success",
        message: sprintf(
          __("The plugin %s has been activated.", "content-blocks-builder"),
          name
        ),
      },
      {
        type: "success",
        message: __("Reloading the page.", "content-blocks-builder"),
      },
    ]);

    // Reload the modal
    reload();
  };

  return (
    <>
      <TemplateItemStyled
        className={classnames("template-item", {
          "is-ready": canInsert,
          "is-pro": isPro,
          "has-pro-features": hasProFeatures,
          "has-missing-blocks": !!missingBlocks?.length,
          "require-pro": !isPremium && isPro,
          "is-loading-data": isItemLoadingData,
          "is-inserting": modalState?.insertingPattern === pattern.id,
        })}
      >
        {isItemLoadingData && <Spinner />}
        {itemPreview}
        {modalState?.insertingPattern === pattern.id && <Spinner />}
        <div className="template-item__footer">
          <div className="template-item__title-details">
            <h3 className="template-item__title">{title}</h3>
            {pattern?.slug && (
              <a
                href={`${getPatternLibraryURL()}/pattern/${
                  pattern?.slug
                }?utm_source=CBB&utm_campaign=pattern-inserter&utm_medium=link&utm_content=view-pattern`}
                className="template-item__more"
                target="_blank"
              >
                {__("View details", "content-blocks-builder")}
              </a>
            )}
          </div>

          <ItemLinks
            links={tutorials}
            label={__("Tutorials:", "content-blocks-builder")}
          />

          <ItemLinks
            links={externalResources}
            label={__("Resources:", "content-blocks-builder")}
          />

          <DependentItems
            items={customBlocks}
            label={__("Custom blocks:", "content-blocks-builder")}
            type="block"
          />

          <DependentItems
            items={customVariations}
            label={__("Custom variations:", "content-blocks-builder")}
            type="variation"
          />

          <ItemLinks
            links={dependentPlugins.map(({ slug, name: title }) => ({
              url: `https://wordpress.org/plugins/${slug}`,
              title,
            }))}
            label={__("Dependencies:", "content-blocks-builder")}
          />

          <ItemDependentPlugins
            {...{
              missingPlugins,
              inactivePlugins,
              libraryState: modalState,
              updateLibraryState: updateModalState,
              onPluginInstalled,
              onPluginActivated,
            }}
          />
        </div>
      </TemplateItemStyled>
    </>
  );
};

const reloadModal = (
  modalState,
  setSavedModalState,
  updateModalState,
  delay = 500
) => {
  updateModalState({ isReloading: true });
  setTimeout(() => {
    // Save current state
    setSavedModalState(modalState);

    const searchParams = new SearchParams();
    searchParams.set("bb-template-modal", 1);
    searchParams.reload();
  }, delay);
};

const TemplateGridControl = ({
  items,
  allItems,
  onChangePage,
  currentPage = 1,
  pageSize = 6,
  TemplateItemControl,
}) => {
  if (!allItems.length) {
    return (
      <div className="template-list-not-found">
        {__("There is no items found.", "content-blocks-builder")}
      </div>
    );
  }

  return (
    <>
      <TemplateGridStyled className="template-list">
        {items.map((item) => (
          <TemplateItemControl item={item} key={item.id} />
        ))}
      </TemplateGridStyled>
      <PaginateLinks
        current={currentPage}
        total={Math.ceil(allItems.length / pageSize)}
        onChange={onChangePage}
      />
    </>
  );
};

export default TemplateGridControl;
