/**
 * External dependencies
 */
import classnames from "classnames";
import { noop, map, isUndefined } from "lodash";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { Spinner, SearchControl, Notice } from "@wordpress/components";
import { useEffect, useRef, useReducer, useMemo } from "@wordpress/element";
import { getBlockTypes } from "@wordpress/blocks";
import { useDispatch, useSelect } from "@wordpress/data";
import { useEntityRecords } from "@wordpress/core-data";

/**
 * Internal dependencies
 */
import { useLocalStorage } from "sdk/utils";
import {
  getIsResolving,
  getHasFinishedResolution,
  useInsertionPoint,
  userCanManagePlugins,
} from "../../utils";
import { store as boldblocksDataStore } from "../../store";
import SortControl from "./sort-control";
import KeywordsControl from "./keywords-control";
import HeaderActionsControl from "./header-actions-control";
import TemplateGridControl, { TemplateItemControl } from "./grid-control";
import { ModalStyled } from "./styles";
import {
  ModalContext,
  initialState,
  reducer,
  dispatchModalState,
  useModalContent,
  cachedModalStateKey,
  getPatternLibraryURL,
} from "./data";

/**
 * Add template from the library
 *
 * @param {TemplatesModalProps} props Component props.
 *
 * @return {WPComponent}.
 */
export function TemplatesModal({
  isModalOpen = false,
  setIsModalOpen,
  onCancel = noop,
  className,
}) {
  const {
    getPlugins,
    getPatterns,
    getPatternKeywords,
    getPatternsModalState,
  } = useSelect((select) => select(boldblocksDataStore), []);

  const { setPatternsModalState } = useDispatch(boldblocksDataStore);

  // Query plugins data
  const canManagePlugins = userCanManagePlugins();

  let plugins = getPlugins();
  const isLoadingPlugins = getIsResolving("getPlugins");
  let isFinishedLoadingPlugins = getHasFinishedResolution("getPlugins");
  if (!isUndefined(canManagePlugins) && !canManagePlugins) {
    isFinishedLoadingPlugins = true;
  }

  // Query patterns
  const patternData = getPatterns();
  const isLoadingPatterns = getIsResolving("getPatterns");
  const isFinishedLoadingPatterns = getHasFinishedResolution("getPatterns");

  // Query pattern keywords
  const patternKeywords = getPatternKeywords();
  const isLoadingKeywords = getIsResolving("getPatternKeywords");

  // Modal state
  const [savedModalState, setSavedModalState] = useLocalStorage(
    cachedModalStateKey
  );

  const [modalState, dispatch] = useReducer(reducer, initialState, () => {
    const modalState = Object.assign(
      initialState,
      getPatternsModalState(),
      savedModalState ?? {}
    );

    // Clear cache
    setSavedModalState(null);

    return modalState;
  });

  const {
    sortType,
    selectedKeywords,
    searchText,
    isOpenHelp,
    currentPage = 1,
    pageSize = 6,
  } = modalState;

  const updateModalState = (payload) => dispatchModalState(dispatch, payload);

  // Build data for grid
  const [fullPatterns, filteredPatterns] = useModalContent(
    modalState,
    patternData
  );

  // Get grid loading status
  const isLoadingGrid =
    isLoadingPatterns || isLoadingPlugins || isUndefined(canManagePlugins);
  const isFinishedLoadingGrid =
    isFinishedLoadingPatterns && isFinishedLoadingPlugins;

  // Focus on search box
  const searchRef = useRef(null);
  useEffect(() => {
    if (isModalOpen) {
      searchRef.current && searchRef.current.focus();
    }
  }, [isModalOpen]);

  // Handle modal close event.
  const closeModal = () => {
    setIsModalOpen(false);

    // Save state to the store
    setPatternsModalState({ ...modalState, insertingPattern: "" });

    onCancel();
  };

  // Insert handler
  const [, onInsertPattern] = useInsertionPoint({ shouldFocusBlock: true });
  const insertPatternHandler = (pattern, blocks) => {
    onInsertPattern(pattern, blocks);

    updateModalState({ insertingPattern: "" });

    // Submit download
    fetch(`${getPatternLibraryURL()}/wp-json/boldblocks/v1/submitDownload`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ id: pattern.id }),
      credentials: "omit",
    });

    closeModal();
  };

  // Query available block names
  const availableBlockTypes = getBlockTypes();
  const availableBlockNames = map(availableBlockTypes, "name");

  const {
    records: localBlocks,
    isResolving: isResolvingLocalBlocks,
  } = useEntityRecords("postType", "boldblocks_block", { per_page: 100 });

  const {
    records: localVariations,
    isResolving: isResolvingLocalVariations,
  } = useEntityRecords("postType", "boldblocks_variation", { per_page: 100 });

  // Memorized context value
  const contextValue = useMemo(() => {
    return {
      modalState,
      updateModalState,
      availableBlockNames,
      plugins,
      canManagePlugins,
      onInsertPattern: insertPatternHandler,
      localBlocks,
      isResolvingLocalBlocks,
      localVariations,
      isResolvingLocalVariations,
    };
  }, [
    modalState,
    updateModalState,
    availableBlockNames,
    plugins,
    canManagePlugins,
    insertPatternHandler,
    localBlocks,
    isResolvingLocalBlocks,
    localVariations,
    isResolvingLocalVariations,
  ]);

  return (
    <>
      <ModalContext.Provider value={contextValue}>
        <ModalStyled
          onRequestClose={() => closeModal()}
          className={classnames("template-modal", className, {
            "is-locked":
              modalState?.installingPlugins?.length ||
              modalState?.activatingPlugins?.length ||
              modalState?.insertingPattern ||
              modalState?.isReloading,
          })}
          isFullScreen
          __experimentalHideHeader={true}
        >
          <div className="template-header">
            <SearchControl
              className="search-box"
              label={__("Search...", "content-blocks-builder")}
              value={searchText}
              onChange={(searchText) => {
                if (searchText.length >= 3) {
                  updateModalState({ searchText, currentPage: 1 });
                } else {
                  updateModalState({ searchText });
                }
              }}
              ref={searchRef}
            />
            <div className="sort-box">
              <SortControl
                onChange={(value) => {
                  updateModalState({
                    sortType: value,
                    currentPage: 1,
                  });
                }}
                value={sortType}
              />
            </div>
            <KeywordsControl
              items={
                filteredPatterns.length !== patternData.length
                  ? filteredPatterns
                  : []
              }
              keywords={patternKeywords}
              isLoading={isLoadingKeywords}
              selectedKeywords={selectedKeywords}
              onChange={(newKeywords) => {
                updateModalState({
                  selectedKeywords: newKeywords,
                  currentPage: 1,
                });
              }}
            />
            <HeaderActionsControl
              isOpenHelp={isOpenHelp}
              onToggleHelp={() => {
                updateModalState({ isOpenHelp: !isOpenHelp });
              }}
              onCloseModal={closeModal}
            />
          </div>
          {isOpenHelp && (
            <Notice
              status="info"
              className="template-modal__legend"
              isDismissible={false}
            >
              <p>
                <strong>
                  {__(
                    "Click on the pattern preview to insert.",
                    "content-blocks-builder"
                  )}
                </strong>
                <span>
                  {__(
                    " Click on the view details link on each pattern to see how it work in the real world.",
                    "content-blocks-builder"
                  )}
                </span>
              </p>
              <p>
                {__(
                  "Pro items require the Pro version to insert. Free items with Pro features like parallax, animations... require the Pro version to work full-functional. They will still work perfectly fine on the Free version but without Pro features.",
                  "content-blocks-builder"
                )}
              </p>
              <p>
                {__(
                  "Some items require blocks, variations from the library. You have to import those dependent blocks, variations into your site to use those items.",
                  "content-blocks-builder"
                )}
              </p>
              <p>
                {__(
                  "Some items require blocks in external plugins. You have to install and activate those required plugins to use those items. All external plugins included in patterns are developed and maintained by us, so they are safe to use.",
                  "content-blocks-builder"
                )}
              </p>
            </Notice>
          )}
          {isLoadingGrid && <Spinner />}
          <div className="template-list-wrapper scrollbar">
            {isFinishedLoadingGrid && (
              <>
                <TemplateGridControl
                  items={fullPatterns}
                  allItems={filteredPatterns}
                  onChangePage={(i) => {
                    updateModalState({ currentPage: i });
                  }}
                  currentPage={currentPage}
                  pageSize={pageSize}
                  TemplateItemControl={TemplateItemControl}
                />
              </>
            )}
          </div>
        </ModalStyled>
      </ModalContext.Provider>
    </>
  );
}
