/**
 * External dependencies
 */
import classnames from "classnames";

/**
 * WordPress dependencies
 */
import { __, _x } from "@wordpress/i18n";
import { blockDefault } from "@wordpress/icons";
import {
  store as blocksStore,
  registerBlockType,
  getCategories,
  setCategories,
} from "@wordpress/blocks";
import { useSelect } from "@wordpress/data";
import { applyFilters } from "@wordpress/hooks";
import {
  store as blockEditorStore,
  InnerBlocks,
  useBlockProps,
  useInnerBlocksProps,
} from "@wordpress/block-editor";
import { Disabled } from "@wordpress/components";
import { Fragment } from "@wordpress/element";

/**
 * Internal dependencies
 */
import { InlineSVG } from "sdk/components";
import { log } from "sdk/utils";
import Placeholder from "./placeholder";
import {
  buildTemplateFromBlocks,
  sanitizeClassString,
  limitSVGDimension,
  buildExampleInnerBlocksFromBlocks,
  getLayout,
  getCoreFeatures,
  CBBCoreBlockSupports,
} from "../utils";
import deprecatedData from "./deprecated";
import buildTransforms from "./transforms";

/**
 * Register content blocks
 */
// Add BoldBlocks category
setCategories([
  {
    slug: "boldblocks",
    title: __("Custom Blocks", "content-blocks-builder"),
  },
  ...getCategories(),
]);
const { blocks: customBlocks } = BoldBlocksBlocks || {};
if (customBlocks && customBlocks.length) {
  customBlocks.forEach(
    ({
      id,
      name,
      parentName,
      title,
      blocks: rawBlocks,
      description,
      templateLock = "all",
      standalone,
      parent,
      className,
      blockIcon,
      blockSupports: boldblocksSupports = {},
      blockAttributes = [],
      enableVariationPicker,
      blockVersion,
      isReadonly = false,
      isTransformable = false,
    }) => {
      let blocks;
      try {
        blocks = JSON.parse(rawBlocks);
      } catch (error) {
        log(error, "error");
      }

      if (templateLock === "inherit") {
        templateLock = undefined;
      } else if (!blocks.length) {
        blocks = [];
        // Clear template lock for empty blocks
        templateLock = false;
      }

      const template = buildTemplateFromBlocks(blocks);

      const exampleInnerBlocks = buildExampleInnerBlocksFromBlocks(blocks);

      // Layout type
      const layoutType = parent ? parent["layoutType"] ?? "vstack" : false;
      const layoutItem = layoutType ? `${layoutType}Item` : "standalone";

      const coreSpacing = getCoreFeatures(layoutItem);
      const parentCoreSpacing = layoutType ? getCoreFeatures(layoutType) : {};

      let blockInfo = {
        id,
        name,
        parentName,
        template,
        templateLock: templateLock ? templateLock : "None",
        layoutType: layoutItem,
      };

      if (isReadonly) {
        templateLock = false;
      }

      const sharedSupports = {
        blockVersion,
        copyPaste: true,
        devicePreview: true,
      };

      // Add default features.
      boldblocksSupports = {
        ...boldblocksSupports,
        ...sharedSupports,
        blockAttributes,
        blockInfo,
      };

      const blockSupports = {
        ...CBBCoreBlockSupports,
        ...coreSpacing,
        filter: {
          duotone: boldblocksSupports?.background,
          __experimentalDefaultControls: { duotone: false },
        },
        align:
          layoutItem === "standalone" || layoutItem === "vstackItem"
            ? ["wide", "full"]
            : false,
        layout: layoutItem !== "accordionItem",
        boldblocks: {
          ...boldblocksSupports,
          layoutType: layoutItem,
          syncContent: true,
        },
      };

      // Make grid/carousel/accordion item only nested inside the parent block.
      if (
        layoutType === "grid" ||
        layoutType === "carousel" ||
        layoutType === "accordion"
      ) {
        standalone = false;
      }

      const blockClass = classnames(
        "wp-block-boldblocks-custom",
        { [`is-${layoutType}-item`]: layoutType, "has-parent": parent },
        sanitizeClassString(className)
      );

      const renderBlockInner = (
        layoutType,
        children,
        props = {},
        isEdit = false
      ) => {
        if (layoutType === "carousel") {
          return <div className="carousel__inner">{children}</div>;
        } else if (layoutType === "accordionItem") {
          const {
            attributes: {
              boldblocks: { accordionItem: { id = "" } = {} } = {},
            } = {},
          } = props;
          return (
            <div
              id={`c-${id}`}
              className={classnames("accordion-collapse", {
                collapse: !isEdit,
              })}
            >
              <div className="accordion-body">{children}</div>
            </div>
          );
        }

        return children;
      };

      // Separate save function to re-use it in deprecated.
      const saveChildBlock = (supports = {}) => (props) => {
        return (
          <div
            {...useBlockProps.save({
              className: blockClass,
            })}
          >
            {applyFilters("boldblocks.save.blockInner", null, {
              ...props,
              name,
              supports,
            })}
            {isReadonly
              ? null
              : renderBlockInner(
                  `${layoutType}Item`,
                  <InnerBlocks.Content />,
                  props
                )}
          </div>
        );
      };

      const blockSettings = {
        apiVersion: 3,
        category: "boldblocks",
        title,
        description,
        icon: blockIcon ? (
          <InlineSVG markup={blockIcon} sanitizeAttribute={limitSVGDimension} />
        ) : (
          blockDefault
        ),
        supports: blockSupports,
        usesContext: ["boldblocks/boldblocks", "postId", "postType"],
        attributes: { boldblocks: { type: "object", default: {} } },
        example: { innerBlocks: exampleInnerBlocks },
        deprecated: deprecatedData.map((item) => ({
          ...item,
          attributes: { boldblocks: { type: "object", default: {} } },
          supports: { ...blockSupports, ...(item?.supports ?? {}) },
          save: saveChildBlock(item?.supports ?? {}),
        })),
        transforms: isTransformable
          ? buildTransforms(
              name,
              layoutType ? `${layoutType}Item` : "standalone"
            )
          : null,
        edit(props) {
          const blockProps = useBlockProps({
            className: blockClass,
          });

          const { hasInnerBlocks } = useSelect(
            (select) => {
              const { getBlocks } = select(blockEditorStore);
              return {
                hasInnerBlocks: !!getBlocks(props.clientId).length,
              };
            },
            [props.clientId]
          );

          const { children, ...innerBlocksProps } = useInnerBlocksProps(
            blockProps,
            {
              template,
              templateLock,
              renderAppender: templateLock
                ? false
                : hasInnerBlocks
                ? undefined
                : InnerBlocks.ButtonBlockAppender,
              layout: getLayout(`${layoutType}Item`, props, false),
            }
          );

          const hasVariations = useSelect(
            (select) => select(blocksStore).getBlockVariations(name).length > 0,
            [name]
          );

          if (enableVariationPicker && hasVariations && !hasInnerBlocks) {
            return <Placeholder {...props} allowSkip={true} />;
          }
          const Wrapper = isReadonly ? Disabled : Fragment;
          const wrapperProps = isReadonly ? { isDisabled: true } : {};
          return (
            <>
              {applyFilters("boldblocks.edit.blockEditInner", null, props)}
              <div {...innerBlocksProps}>
                <Wrapper {...wrapperProps}>
                  {applyFilters(
                    "boldblocks.edit.blockInner",
                    null,
                    props,
                    true
                  )}
                  {renderBlockInner(`${layoutType}Item`, children, props, true)}
                </Wrapper>
              </div>
            </>
          );
        },
        save: saveChildBlock(),
      };

      if (parent && !standalone) {
        blockSettings.parent = [parent.name];
      }

      registerBlockType(name, blockSettings);

      // Register the repeater block if neccessary.
      if (parent) {
        const {
          title: parentTitle,
          description: parentDescription,
          nestedItemCount,
          blockIcon: parentBlockIcon,
          blockSupports: parentBoldBlocksSupports = {},
          blockAttributes: parentBlockAttributes = [],
          enableVariationPicker: parentEnableVariationPicker,
        } = parent;

        const { className: parentClassName = "" } = parent;

        // Sanizite class string
        const parentBlockClass = classnames(
          "wp-block-boldblocks-custom-parent",
          { [`is-${layoutType}`]: layoutType },
          sanitizeClassString(parentClassName)
        );

        // Separate save function to re-use it in deprecated.
        const saveParentBlock = (supports = {}) => (props) => {
          return (
            <div
              {...useBlockProps.save({
                className: parentBlockClass,
              })}
            >
              <>
                {applyFilters("boldblocks.save.blockInner", null, {
                  ...props,
                  name: parentName,
                  supports,
                })}
                {renderBlockInner(layoutType, <InnerBlocks.Content />)}
              </>
            </div>
          );
        };

        let parentBlockInfo = {
          id,
          name: parentName,
          childName: name,
          templateLock: templateLock ? templateLock : "None",
          layoutType,
        };

        const parentBlockSupports = {
          ...CBBCoreBlockSupports,
          ...parentCoreSpacing,
          filter: {
            duotone: parentBoldBlocksSupports?.background,
            __experimentalDefaultControls: { duotone: false },
          },
          align: ["wide", "full"],
          layout: layoutType === "vstack",
          boldblocks: {
            ...parentBoldBlocksSupports,
            ...sharedSupports,
            blockAttributes: parentBlockAttributes,
            layoutType,
            blockInfo: parentBlockInfo,
          },
        };

        const parentBlockSettings = {
          apiVersion: 3,
          category: "boldblocks",
          title: parentTitle,
          description: parentDescription,
          icon: parentBlockIcon ? (
            <InlineSVG
              markup={parentBlockIcon}
              sanitizeAttribute={limitSVGDimension}
            />
          ) : (
            blockDefault
          ),
          supports: parentBlockSupports,
          providesContext: {
            "boldblocks/boldblocks": "boldblocks",
          },
          usesContext: ["postId", "postType"],
          attributes: { boldblocks: { type: "object", default: {} } },
          example: {
            innerBlocks: [
              {
                name,
                innerBlocks: exampleInnerBlocks,
              },
            ],
          },
          deprecated: deprecatedData.map((item) => ({
            ...item,
            attributes: { boldblocks: { type: "object", default: {} } },
            supports: { ...parentBlockSupports, ...(item?.supports ?? {}) },
            save: saveParentBlock(item?.supports ?? {}),
          })),
          transforms: isTransformable
            ? buildTransforms(parentName, layoutType, name)
            : null,
          edit(props) {
            let parentTemplate = [];
            const hasFixedNestedItems =
              nestedItemCount && parseInt(nestedItemCount, 10) > 0;
            if (hasFixedNestedItems) {
              for (
                let i = 0, count = parseInt(nestedItemCount);
                i < count;
                i++
              ) {
                parentTemplate.push([name]);
              }
            } else {
              parentTemplate = [[name]];
            }

            const hasVariations = useSelect(
              (select) =>
                select(blocksStore).getBlockVariations(parentName).length > 0,
              [parentName]
            );

            const blockProps = useBlockProps({
              className: parentBlockClass,
            });

            const { hasInnerBlocks } = useSelect(
              (select) => {
                const { getBlocks } = select(blockEditorStore);
                return {
                  hasInnerBlocks: !!getBlocks(props.clientId).length,
                };
              },
              [props.clientId]
            );

            const { children, ...innerBlocksProps } = useInnerBlocksProps(
              blockProps,
              {
                template: parentTemplate,
                allowedBlocks: [name],
                templateLock: hasFixedNestedItems ? "insert" : false,
                orientation:
                  layoutType === "grid" || layoutType === "carousel"
                    ? "horizontal"
                    : null,
                renderAppender: hasFixedNestedItems
                  ? false
                  : hasInnerBlocks
                  ? undefined
                  : InnerBlocks.ButtonBlockAppender,
                layout: getLayout(layoutType, props, true),
              }
            );

            if (
              parentEnableVariationPicker &&
              !hasInnerBlocks &&
              !hasFixedNestedItems &&
              hasVariations
            ) {
              return <Placeholder {...props} allowSkip={true} />;
            }

            return (
              <>
                {applyFilters("boldblocks.edit.blockEditInner", null, props)}
                <div {...innerBlocksProps}>
                  <>
                    {applyFilters(
                      "boldblocks.edit.blockInner",
                      null,
                      props,
                      true
                    )}
                    {renderBlockInner(layoutType, children, props, true)}
                  </>
                </div>
              </>
            );
          },
          save: saveParentBlock(),
        };

        registerBlockType(parentName, parentBlockSettings);
      }
    }
  );
}
