/**
 * External dependencies
 */
import { find, map, isArray, isUndefined, isEmpty, omit, pick } from "lodash";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { useSelect, useDispatch } from "@wordpress/data";
import { useEntityProp } from "@wordpress/core-data";
import { registerPlugin } from "@wordpress/plugins";
import {
  PluginSidebar,
  PluginSidebarMoreMenuItem,
  PluginDocumentSettingPanel,
  store as editPostStore,
} from "@wordpress/edit-post";
import {
  PanelBody,
  SelectControl,
  ToggleControl,
  TextControl,
  TextareaControl,
  ExternalLink,
} from "@wordpress/components";
import { store as editorStore } from "@wordpress/editor";
import { useEffect, useMemo, useRef } from "@wordpress/element";
import apiFetch from "@wordpress/api-fetch";
import {
  rawHandler,
  unregisterBlockType,
  getBlockTypes,
  hasBlockSupport,
} from "@wordpress/blocks";
import { applyFilters } from "@wordpress/hooks";
import { store as noticesStore } from "@wordpress/notices";

/**
 * Internal dependencies
 */
import { BlockIcon } from "../components/block-icon";
import {
  BoldBlocksFeatures,
  ParentBlockFeatures,
  PremiumFeatures,
  AccordionFeatures,
  getPostBlocksData,
  isPremium,
  getDependentBlockTypes,
  hasMissingBlocks,
  getEditPostURL,
  getNoticeActions,
  userCanManagePlugins,
  refineCustomCSS,
} from "../utils";
import BlockSupports from "../components/block-supports";
import { BlockAttributes } from "../components/custom-attributes";
import { NestedPanelBody } from "sdk/components";
import { log } from "sdk/utils";
import { store as boldblocksDataStore } from "../store";
import { updateDynamicStyle } from "../utils/editor";

/**
 * Update blocks metadata
 */
apiFetch.use((options, next) => {
  const { method, path } = options;
  if (method === "PUT" && path.indexOf("/wp/v2/boldblocks-blocks") === 0) {
    const { data = {}, data: { content, meta = {} } = {} } = options;
    const { changedFields = [] } = meta;
    if (!content && !changedFields?.length) {
      return next(options);
    }

    let changedMeta = pick(meta, changedFields);

    if (content) {
      const blocks = getPostBlocksData(rawHandler({ HTML: content }));
      const boldblocks_block_blocks = JSON.stringify(blocks);

      // Update metadata
      if (meta?.boldblocks_block_blocks !== boldblocks_block_blocks) {
        changedMeta = {
          ...changedMeta,
          boldblocks_block_blocks,
          boldblocks_block_dependent_blocks:
            getDependentBlockTypes(blocks ?? []) ?? [],
        };
      }
    }
    if (!isEmpty(changedMeta)) {
      log("Update blocks metadata");
      log(changedMeta);
      options = { ...options, data: { ...data, meta: changedMeta } };
    }
  }
  return next(options);
});

/**
 * Register the custom content block meta.
 */
registerPlugin("boldblocks-content-block-meta", {
  render: () => {
    const { getCurrentPostType, getEditedPostSlug } = useSelect(
      (select) => select(editorStore),
      []
    );

    // Only display block settings for boldblocks_block content type
    if (getCurrentPostType() !== "boldblocks_block") {
      return null;
    }

    // Get meta, setMeta
    const [savedMeta, setMeta] = useEntityProp(
      "postType",
      "boldblocks_block",
      "meta"
    );

    const changedMetaRef = useRef([]);
    const metaRef = useRef(savedMeta);

    const updateMeta = (name) => (newValue) => {
      if (!changedMetaRef.current.includes(name)) {
        changedMetaRef.current.push(name);
      }
      metaRef.current = {
        ...metaRef.current,
        [name]: newValue,
        changedFields: changedMetaRef.current,
      };
      setMeta({ ...metaRef.current });
    };

    // Template lock options
    const templateLockOptions = [
      { label: __("None", "content-blocks-builder"), value: "false" },
      { label: __("All", "content-blocks-builder"), value: "all" },
      { label: __("Insert", "content-blocks-builder"), value: "insert" },
      {
        label: __("Content Only", "content-blocks-builder"),
        value: "contentOnly",
      },
    ];

    // Repeater block layout options
    const layoutTypeOptions = [
      {
        value: "vstack",
        label: __("Default (vertical stack)", "content-blocks-builder"),
      },
      { value: "grid", label: __("Grid", "content-blocks-builder") },
      { value: "carousel", label: __("Carousel", "content-blocks-builder") },
      { value: "accordion", label: __("Accordion", "content-blocks-builder") },
    ];

    const {
      boldblocks_template_lock: templateLock,
      boldblocks_block_description: blockDescription,
      boldblocks_block_class: blockClasses = "",
      boldblocks_block_icon: blockIcon = "",
      boldblocks_block_supports: blockSupports,
      boldblocks_enable_repeater: enableRepeater = false,
      boldblocks_parent_layout_type: layoutType = "vstack",
      boldblocks_parent_block_icon: parentBlockIcon = "",
      boldblocks_parent_block_supports: parentBlockSupports,
      boldblocks_disable_standalone: disableStandalone = false,
      boldblocks_enable_variation_picker: enableVariationPicker = false,
      boldblocks_parent_block_title: parentTitle,
      boldblocks_parent_block_description: parentDescription,
      boldblocks_is_fixed_nested_item_count: isFixedNestedItemCount = false,
      boldblocks_nested_item_count: nestedItemCount = 1,
      boldblocks_parent_block_class: parentClasses = "",
      boldblocks_parent_enable_variation_picker: parentEnableVariationPicker = false,
      boldblocks_block_external_scripts: externalScripts = [],
      boldblocks_block_custom_scripts: customScripts = [],
      boldblocks_block_external_styles: externalStyles = [],
      boldblocks_block_custom_styles: customStyles = [],
      boldblocks_block_attributes: blockAttributes = [],
      boldblocks_parent_block_attributes: parentBlockAttributes = [],
      boldblocks_enable_custom_attributes: enableCustomAttributes = false,
      boldblocks_parent_enable_custom_attributes: enableParentCustomAttributes = false,
      boldblocks_hide_on_frontend: hideOnFrontend = false,
      boldblocks_is_readonly: isReadonly = false,
      boldblocks_is_transformable: isTransformable = false,
      boldblocks_is_hidden: isHidden = false,
      boldblocks_block_parent: blockParent = "",
      boldblocks_block_ancestor: blockAncestor = "",
      boldblocks_block_dependent_blocks: dependentBlocks,
    } = metaRef.current;

    // Handle supports for repeater enable blocks
    let boldblocksFeatures = { ...BoldBlocksFeatures };
    if (enableRepeater) {
      boldblocksFeatures = omit(boldblocksFeatures, [
        "justifyAlignment",
        "toggle",
      ]);

      if (layoutType === "grid") {
        boldblocksFeatures = omit(boldblocksFeatures, ["verticalAlignment"]);
      }
    }

    // Supported features
    // For child blocks
    let childBlockFeatures = applyFilters("boldblocks.customBlocks.features", {
      ...boldblocksFeatures,
    });

    // For parent blocks
    let parentBlockFeatures = applyFilters(
      "boldblocks.customBlocks.parentFeatures",
      {
        ...omit(boldblocksFeatures, ["aspectRatio"]),
        ...ParentBlockFeatures,
      }
    );

    // Ignore features for specific layout types
    if (enableRepeater) {
      if (layoutType === "accordion") {
        childBlockFeatures = pick(childBlockFeatures, AccordionFeatures);
        parentBlockFeatures = pick(parentBlockFeatures, AccordionFeatures);
      } else if (layoutType !== "grid") {
        childBlockFeatures = omit(childBlockFeatures, ["sticky"]);
      }
    }

    const updateSupportFeature = (fieldName, fieldValue, featureName) => (
      featureValue
    ) => {
      updateMeta(fieldName)({ ...fieldValue, [featureName]: featureValue });
    };

    const restrictBlockSupports = (blockSupports, availableFeatures) => {
      return Object.keys(blockSupports).reduce((previous, current) => {
        let value = blockSupports[current];
        if (!Object.keys(availableFeatures).includes(current)) {
          value = false;
        }
        return { ...previous, [current]: value };
      }, {});
    };

    const updateBlockFeatures = (enableRepeater, layoutType) => {
      if (enableRepeater) {
        updateMeta("boldblocks_block_supports")(
          restrictBlockSupports(blockSupports, childBlockFeatures)
        );

        updateMeta("boldblocks_parent_block_supports")(
          restrictBlockSupports(parentBlockSupports, parentBlockFeatures)
        );
      }
    };

    const pluginTitle = __("Content block settings", "content-blocks-builder");

    const PluginControls = (
      <>
        <SelectControl
          label={__("Template lock", "content-blocks-builder")}
          options={templateLockOptions}
          onChange={updateMeta("boldblocks_template_lock")}
          value={templateLock}
        />
        <TextareaControl
          label={__("Block description", "content-blocks-builder")}
          value={blockDescription}
          onChange={updateMeta("boldblocks_block_description")}
          rows={4}
        />
        <TextControl
          autoComplete="off"
          label={__("Block CSS class(es)", "content-blocks-builder")}
          value={blockClasses}
          onChange={updateMeta("boldblocks_block_class")}
          help={__(
            "Separate multiple classes with spaces.",
            "content-blocks-builder"
          )}
        />
        <BlockIcon
          label={__("Block icon:", "content-blocks-builder")}
          value={blockIcon}
          onChange={updateMeta("boldblocks_block_icon")}
        />
        <BlockSupports
          label={__("Extended block supports:", "content-blocks-builder")}
          name="boldblocks_block_supports"
          value={blockSupports}
          features={childBlockFeatures}
          onChange={updateSupportFeature}
          premiumFeatures={PremiumFeatures}
          isPremium={isPremium}
        />
        {blockSupports?.customAttributes && (
          <BlockAttributes
            attributes={blockAttributes}
            onChangeAttributes={updateMeta("boldblocks_block_attributes")}
            enableCustomAttributes={enableCustomAttributes}
            onChangeEnableCustomAttributes={updateMeta(
              "boldblocks_enable_custom_attributes"
            )}
            label={__("Block custom attributes", "content-blocks-builder")}
          />
        )}
        <ToggleControl
          label={__(
            "Show the block variation picker if it has some",
            "content-blocks-builder"
          )}
          checked={enableVariationPicker}
          onChange={updateMeta("boldblocks_enable_variation_picker")}
        />
        <TextControl
          autoComplete="off"
          label={__("Parent block(s)", "content-blocks-builder")}
          value={blockParent}
          onChange={updateMeta("boldblocks_block_parent")}
          help={
            <>
              {__(
                "The block is only available when nested within parent block(s). Separate multiple values with comma.",
                "content-blocks-builder"
              )}{" "}
              <ExternalLink href="https://developer.wordpress.org/block-editor/reference-guides/block-api/block-metadata/#parent">
                {__("Learn more", "content-blocks-builder")}
              </ExternalLink>
            </>
          }
          placeholder="core/group,boldblocks/group"
        />
        <TextControl
          autoComplete="off"
          label={__("Ancestor block(s)", "content-blocks-builder")}
          value={blockAncestor}
          onChange={updateMeta("boldblocks_block_ancestor")}
          help={
            <>
              {__(
                "The block is only available when nested within the subtree of ancestor block(s). Separate multiple values with comma.",
                "content-blocks-builder"
              )}{" "}
              <ExternalLink href="https://developer.wordpress.org/block-editor/reference-guides/block-api/block-metadata/#ancestor">
                {__("Learn more", "content-blocks-builder")}
              </ExternalLink>
            </>
          }
          placeholder="core/group,boldblocks/group"
        />
        <ToggleControl
          label={__("Is readonly", "content-blocks-builder")}
          checked={isReadonly}
          onChange={updateMeta("boldblocks_is_readonly")}
          help={__(
            "This setting makes the block to behave like a synced pattern. Any page or post that uses this block will receive the updated version of its inner content. However, you can still modify the style of the wrapper block.",
            "content-blocks-builder"
          )}
        />
        <ToggleControl
          label={__("Is transformable", "content-blocks-builder")}
          checked={isTransformable}
          onChange={updateMeta("boldblocks_is_transformable")}
          help={__(
            "This setting allows you to convert other blocks into this block.",
            "content-blocks-builder"
          )}
        />
        <ToggleControl
          label={__("Is hidden", "content-blocks-builder")}
          checked={isHidden}
          onChange={updateMeta("boldblocks_is_hidden")}
          help={__(
            "This setting allows you to hide the block from the inserter.",
            "content-blocks-builder"
          )}
        />
        <ToggleControl
          label={__(
            "Hide the block on the front end",
            "content-blocks-builder"
          )}
          checked={hideOnFrontend}
          onChange={updateMeta("boldblocks_hide_on_frontend")}
          help={__(
            "This setting allow you to hide the block from showing on the front end.",
            "content-blocks-builder"
          )}
        />
        <ToggleControl
          label={__(
            "Create a repeater parent block for this block",
            "content-blocks-builder"
          )}
          checked={enableRepeater}
          onChange={(value) => {
            updateMeta("boldblocks_enable_repeater")(value);

            // Update block supports
            updateBlockFeatures(value, layoutType);
          }}
        />
        {enableRepeater && (
          <>
            <SelectControl
              label={__("Parent block layout", "content-blocks-builder")}
              options={layoutTypeOptions}
              onChange={(value) => {
                updateMeta("boldblocks_parent_layout_type")(value);

                // Update block supports
                updateBlockFeatures(enableRepeater, value);
              }}
              value={layoutType}
            />
            <TextControl
              label={__("Parent block title", "content-blocks-builder")}
              value={parentTitle}
              onChange={updateMeta("boldblocks_parent_block_title")}
              help={__(
                "The default value is: 'Repeater: $post_title'",
                "content-blocks-builder"
              )}
            />
            <TextareaControl
              label={__("Parent block description", "content-blocks-builder")}
              value={parentDescription}
              onChange={updateMeta("boldblocks_parent_block_description")}
              rows={4}
            />
            <TextControl
              autoComplete="off"
              label={__("Parent block CSS class(es)", "content-blocks-builder")}
              value={parentClasses}
              onChange={updateMeta("boldblocks_parent_block_class")}
              help={__(
                "Separate multiple classes with spaces.",
                "content-blocks-builder"
              )}
            />
            <BlockIcon
              label={__("Parent block icon", "content-blocks-builder")}
              value={parentBlockIcon}
              onChange={updateMeta("boldblocks_parent_block_icon")}
            />
            <BlockSupports
              label={__(
                "Parent block extended block supports:",
                "content-blocks-builder"
              )}
              name="boldblocks_parent_block_supports"
              value={parentBlockSupports}
              features={parentBlockFeatures}
              premiumFeatures={PremiumFeatures}
              onChange={updateSupportFeature}
              isPremium={isPremium}
            />
            {parentBlockSupports?.customAttributes && (
              <BlockAttributes
                attributes={parentBlockAttributes}
                onChangeAttributes={updateMeta(
                  "boldblocks_parent_block_attributes"
                )}
                enableCustomAttributes={enableParentCustomAttributes}
                onChangeEnableCustomAttributes={updateMeta(
                  "boldblocks_parent_enable_custom_attributes"
                )}
                label={__(
                  "Parent block custom attributes",
                  "content-blocks-builder"
                )}
              />
            )}
            <ToggleControl
              label={__(
                "Show the block variation picker for the parent block if it has some",
                "content-blocks-builder"
              )}
              checked={parentEnableVariationPicker}
              onChange={updateMeta("boldblocks_parent_enable_variation_picker")}
            />
            <ToggleControl
              label={__(
                "Make this repeater block have only a fixed number of child blocks",
                "content-blocks-builder"
              )}
              checked={isFixedNestedItemCount}
              onChange={updateMeta("boldblocks_is_fixed_nested_item_count")}
            />
            {isFixedNestedItemCount && (
              <TextControl
                type="number"
                label={__("Number of child blocks", "content-blocks-builder")}
                value={nestedItemCount}
                onChange={updateMeta("boldblocks_nested_item_count")}
                min={1}
              />
            )}
            {layoutType === "vstack" && (
              <ToggleControl
                label={__(
                  "Restrict the child block so that it's only available as a nested block of this repeater block",
                  "content-blocks-builder"
                )}
                checked={disableStandalone}
                onChange={updateMeta("boldblocks_disable_standalone")}
                help={__(
                  "This setting only apply to the vertical stack layout",
                  "content-blocks-builder"
                )}
              />
            )}
          </>
        )}
        {applyFilters("boldblocks.blockSettings.additionalSettings", null, {
          meta: metaRef.current,
          setMeta: updateMeta,
        })}
      </>
    );

    const manageScriptsTitle = __("Manage scripts", "content-blocks-builder");
    const disabledScriptsTitle = __(
      "Pro customers can manage external JS resources and input custom JS code for blocks",
      "content-blocks-builder"
    );

    const manageStylesTitle = __(
      "Manage stylesheets",
      "content-blocks-builder"
    );
    const disabledStylesTitle = __(
      "Pro customers can manage external style resources and input custom CSS code for blocks",
      "content-blocks-builder"
    );

    const scriptControls = applyFilters(
      "boldblocks.manageBlocks.customScripts",
      null,
      {
        externalScripts,
        customScripts,
        onChangeExternalScripts: (values) =>
          updateMeta("boldblocks_block_external_scripts")(values),
        onChangeCustomScripts: (values) =>
          updateMeta("boldblocks_block_custom_scripts")(values),
      }
    );

    const isHTMLPreview =
      dependentBlocks &&
      dependentBlocks?.length === 1 &&
      dependentBlocks[0] === "core/html";
    const selector = isHTMLPreview
      ? `.wp-block-boldblocks-${getEditedPostSlug()}`
      : ".wp-block-post-content";
    const inlineHandle = "edit-boldblocks-custom-blocks-inline-css";
    const resetEditorCSS =
      ".wp-block-post-content{position:relative!important;top:unset!important;right:unset!important;bottom:unset!important;left:unset!important;display:block!important;width:auto!important;max-width:none!important;height:auto!important;max-height:none!important;}.is-selected,.has-child-selected,.has-child-selected * {animation:none!important;}";

    const styleControls = applyFilters(
      "boldblocks.manageBlocks.customStyles",
      null,
      {
        externalStyles,
        customStyles,
        onChangeExternalStyles: (values) =>
          updateMeta("boldblocks_block_external_styles")(values),
        onChangeCustomStyles: (values) => {
          if (values?.length) {
            let dynamicStyle = values.reduce((prev, { value }) => {
              let formatedValue = refineCustomCSS(value, { selector });
              if (formatedValue) {
                prev = `${prev}${formatedValue}`;
              }

              return prev;
            }, "");

            if (dynamicStyle) {
              dynamicStyle = `${dynamicStyle}${resetEditorCSS}`;
              updateDynamicStyle(
                inlineHandle,
                dynamicStyle,
                false,
                isHTMLPreview
              );
            } else {
              updateDynamicStyle(inlineHandle, "", true, isHTMLPreview);
            }
          } else {
            updateDynamicStyle(inlineHandle, "", true, isHTMLPreview);
          }
          updateMeta("boldblocks_block_custom_styles")(values);
        },
      }
    );

    return (
      <>
        <PluginDocumentSettingPanel
          name="content-block-settings"
          title={pluginTitle}
          initialOpen={true}
        >
          {PluginControls}
          <NestedPanelBody title={manageScriptsTitle} initialOpen={false}>
            {isPremium ? scriptControls : disabledScriptsTitle}
          </NestedPanelBody>
          <NestedPanelBody title={manageStylesTitle} initialOpen={false}>
            {isPremium ? styleControls : disabledStylesTitle}
          </NestedPanelBody>
        </PluginDocumentSettingPanel>
        <PluginSidebarMoreMenuItem target="content-block-settings">
          {pluginTitle}
        </PluginSidebarMoreMenuItem>
        <PluginSidebar name="content-block-settings" title={pluginTitle}>
          <PanelBody>{PluginControls}</PanelBody>
          <PanelBody title={manageScriptsTitle} initialOpen={false}>
            {isPremium ? scriptControls : disabledScriptsTitle}
          </PanelBody>
          <PanelBody title={manageStylesTitle} initialOpen={false}>
            {isPremium ? styleControls : disabledStylesTitle}
          </PanelBody>
        </PluginSidebar>
      </>
    );
  },
});

registerPlugin("boldblocks-prevent-block-nested-in-itself", {
  render: () => {
    const { getCurrentPostType, getCurrentPostId } = useSelect(
      (select) => select(editorStore),
      []
    );

    const hiddenBlockTypes = useSelect(
      (select) => select(editPostStore).getHiddenBlockTypes(),
      []
    );

    const { hideBlockTypes, showBlockTypes } = useDispatch(editPostStore);

    const { blocks: customBlocks = [] } = BoldBlocksBlocks || {};
    let shownBlocks = [];
    let hiddenBlocks = customBlocks.reduce((prev, { name, isHidden }) => {
      if (isHidden) {
        prev.push(name);
      }

      return prev;
    }, []);

    if (getCurrentPostType() === "boldblocks_block") {
      const currentPostId = getCurrentPostId();

      const block = find(customBlocks, ["id", currentPostId]);
      if (block) {
        const { name, parent: { name: parentName } = {} } = block;
        if (name) {
          hiddenBlocks.push(name);
        }

        if (parentName) {
          hiddenBlocks.push(parentName);
        }
      }
    }

    if (hiddenBlockTypes.length) {
      shownBlocks = hiddenBlockTypes.filter(
        (name) =>
          hiddenBlocks.indexOf(name) === -1 && hasBlockSupport(name, "cbb")
      );
    }

    useEffect(() => {
      if (!!shownBlocks.length) {
        showBlockTypes(shownBlocks);
      }

      if (!!hiddenBlocks.length) {
        hideBlockTypes(hiddenBlocks);
      }
    }, [shownBlocks.length, hiddenBlocks.length]);

    return null;
  },
});

registerPlugin("boldblocks-disable-blocks-with-missing-blocks", {
  render: () => {
    const { createWarningNotice } = useDispatch(noticesStore);
    const { getMissingBlockStatus } = useSelect(
      (select) => select(boldblocksDataStore),
      []
    );
    const { loadMissingBlock, setMissingBlockStatus } = useDispatch(
      boldblocksDataStore
    );
    const canManagePlugins = userCanManagePlugins();

    const missingBlocks = useMemo(() => {
      const availableBlockTypes = getBlockTypes();
      const availableBlockNames = map(availableBlockTypes, "name");

      return (window?.BoldBlocksBlocks?.blocks ?? []).reduce(
        (prev, { id, name, title, blocks: rawBlocks, dependentBlocks }) => {
          if (!isArray(dependentBlocks) || !dependentBlocks.length) {
            let blocks;
            try {
              blocks = JSON.parse(rawBlocks);
            } catch (error) {
              log(error, "error");
            }

            if (blocks && isArray(blocks)) {
              dependentBlocks = getDependentBlockTypes(blocks, []);
            }
          }

          if (isArray(dependentBlocks) && !!dependentBlocks.length) {
            const missingBlock = hasMissingBlocks(
              dependentBlocks,
              availableBlockNames
            );
            if (missingBlock) {
              prev.push({
                id,
                name,
                title,
                missingBlock,
              });
            }
          }

          return prev;
        },
        []
      );
    }, []);

    useEffect(() => {
      if (!!missingBlocks.length) {
        missingBlocks.forEach((block) => {
          const { name } = block;
          unregisterBlockType(name);
        });
      }
    }, []);

    useEffect(() => {
      // The variable has been loaded and has some missing blocks
      if (!isUndefined(canManagePlugins) && !!missingBlocks.length) {
        missingBlocks.forEach(async (block) => {
          const { id, name, missingBlock } = block;
          const noticeId = `block-${id}`;
          if (!getMissingBlockStatus(noticeId)) {
            let actions = [
              {
                label: __("Edit block", "content-blocks-builder"),
                url: getEditPostURL(id),
              },
            ];
            const filterValue = `block:${missingBlock}`;

            const noticeMessage = sprintf(
              __(
                "The block %s has been disabled because it contains a missing block %s.",
                "content-blocks-builder"
              ),
              name,
              missingBlock
            );

            log(noticeMessage, "warn");

            let plugin;
            try {
              plugin = await loadMissingBlock(filterValue);
            } catch (error) {
              log(error, "error");
            }

            if (plugin && !isEmpty(plugin)) {
              actions.push(...getNoticeActions(plugin));
            }

            createWarningNotice(noticeMessage, { actions });
            setMissingBlockStatus(noticeId);
          }
        });
      }
    }, [canManagePlugins, missingBlocks.length]);

    return null;
  },
});
