/**
 * External dependencies
 */
import { isArray } from "lodash";
import classnames from "classnames";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { addFilter } from "@wordpress/hooks";
import { hasBlockSupport } from "@wordpress/blocks";

/**
 * Internal dependencies
 */
import { isValidSettingObject, addEditProps, usePropsStyle } from "sdk/utils";
import { boldblocksHasSupport } from "../../utils";

/**
 * Define feature name
 */
const featureName = "animation";

/**
 * Get styles for the feature.
 *
 * @param {Object} animationData
 */
function getAnimationStyle(animationData) {
  let style = {};
  if (isValidSettingObject(animationData)) {
    const {
      animationDuration,
      animationDelay,
      animationIterationCount,
      animationTimingFunction,
    } = animationData;

    style = {
      animationDuration,
      animationDelay,
      animationIterationCount,
    };
    if (animationTimingFunction) {
      style = { ...style, animationTimingFunction };
    }
  }

  return style;
}

/**
 * Get the data for the feature.
 *
 * @param {Object} animationValue
 */
function getFeatureData(animationValue) {
  const { animations = [], animateMultipleTimes = false } = animationValue;

  let animationObject = {};

  // Filter only animation with duration > 0
  const validAnimations =
    isArray(animations) &&
    animations.filter(({ name, duration }) => name && duration);

  // Bail if there is no valid animation
  if (validAnimations.length) {
    // Build animation object
    animationObject = validAnimations.reduce(
      (
        accumulator,
        {
          name,
          duration,
          delay = 0,
          repeat = 1,
          infinite = false,
          timingFunction,
        }
      ) => {
        accumulator.animationName = accumulator.animationName
          ? `${accumulator.animationName},${name}`
          : name;
        accumulator.animationDuration = accumulator.animationDuration
          ? `${accumulator.animationDuration},${duration}s`
          : `${duration}s`;
        accumulator.animationDelay = accumulator.animationDelay
          ? `${accumulator.animationDelay},${delay}s`
          : `${delay}s`;
        repeat = infinite ? "infinite" : repeat;
        accumulator.animationIterationCount = accumulator.animationIterationCount
          ? `${accumulator.animationIterationCount},${repeat}`
          : repeat;

        if (timingFunction) {
          accumulator.animationTimingFunction = accumulator.animationTimingFunction
            ? `${accumulator.animationTimingFunction},${timingFunction}`
            : timingFunction;
        }

        return accumulator;
      },
      {
        animationName: "",
        animationDuration: "",
        animationDelay: "",
        animationIterationCount: "",
        animationTimingFunction: "",
      }
    );

    if (
      animationObject.hasOwnProperty("animationTimingFunction") &&
      !animationObject?.animationTimingFunction
    ) {
      delete animationObject.animationTimingFunction;
    }

    animationObject["animateMultipleTimes"] = animateMultipleTimes;
  }

  return animationObject;
}

/**
 * Override props assigned to save component to inject the CSS variables definition.
 *
 * @param {Object} props      Additional props applied to save element.
 * @param {Object} blockType  Block type.
 * @param {Object} attributes Block attributes.
 *
 * @return {Object} Filtered props applied to save element.
 */
export function addSaveProps(props, blockType, attributes) {
  if (!boldblocksHasSupport(blockType, featureName)) {
    return props;
  }

  const animationObject = (({
    boldblocks: { animations = [], animateMultipleTimes = false } = {},
  }) => ({
    animations,
    animateMultipleTimes,
  }))(attributes);

  let animationData;
  if (attributes?.isBlockEdit) {
    animationData = usePropsStyle({
      value: animationObject,
      getStyle: (value) => () => getFeatureData(value),
    });
  } else {
    animationData = getFeatureData(animationObject);
  }

  if (isValidSettingObject(animationData)) {
    props = {
      ...props,
      "data-reveal-animation": JSON.stringify(animationData),
    };

    if (
      !hasBlockSupport(blockType, "BoldBlocksDeprecatedAnimationV1") &&
      !attributes?.isBlockEdit
    ) {
      props.className = classnames(props.className, "animate__waiting");
    }

    props.style = {
      ...props.style,
      ...getAnimationStyle(animationData),
    };
  }

  return props;
}
addFilter(
  "blocks.getSaveContent.extraProps",
  `boldblocks/${featureName}/addSaveProps`,
  addSaveProps
);

/**
 * Filters registered block settings to extend the block edit wrapper
 * to apply the desired styles and classnames properly.
 *
 * @param {Object} settings Original block settings.
 *
 * @return {Object}.Filtered block settings.
 */
addFilter(
  "blocks.registerBlockType",
  `boldblocks/${featureName}/addEditProps`,
  addEditProps(
    addSaveProps,
    (settings, { featureName }) => boldblocksHasSupport(settings, featureName),
    { featureName }
  )
);
