/**
 * External dependencies
 */
import classnames from "classnames";
import { isEmpty, isObject } from "lodash";
import styled from "@emotion/styled";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { addFilter } from "@wordpress/hooks";
import { createHigherOrderComponent } from "@wordpress/compose";
import {
  BaseControl,
  ToggleControl,
  SelectControl,
  TabPanel,
  Slot,
  Fill,
  __experimentalToolsPanelItem as ToolsPanelItem,
} from "@wordpress/components";
import { store as blockEditorStore } from "@wordpress/block-editor";
import { useSelect } from "@wordpress/data";
import { useEffect } from "@wordpress/element";

/**
 * Internal dependencies
 */
import { PanelBodyInner } from "sdk/components";
import { handleChangeSettingGroupField, addEditProps } from "sdk/utils";
import { boldblocksHasSupport } from "../../utils";
import {
  MEDIA_TYPES,
  IMAGE_BACKGROUND_TYPE,
  VIDEO_BACKGROUND_TYPE,
  useFeaturedMedia,
  hasAnimationSettingValue,
  getAnimationSettingValue,
} from "./utils";
import CustomMediaUpload from "../../components/custom-media-upload";
import BackgroundSettings from "./background-settings-control";

const TabPanelStyled = styled(TabPanel)`
  .components-tab-panel__tabs {
    border-bottom: 1px solid #ddd;
  }

  .components-tab-panel__tab-content {
    padding-top: 8px;
  }
`;

/**
 * Define feature name
 */
const featureName = "background";

/*
{
  mediaType: ["image", "video"];
  image: {
    id, url, alt, customAlt;
  }
  video: {
    id, url, poster;
  }
  settings: {
    focalPoint, // {.5, .5}
    size, // cover, contain, auto
    objectFit, // cover, contain, fill
    repeat, // no-repeat, repeat, repeat-x, repeat-y
    isFixed, // fixed or scroll
    animation: {
      type, // none, parallax, scroll-horizontal, scroll-veritcal
      settings, // {speed: .5}, {duration: 2.5}
    }
  }
}
*/

/**
 * Check whether has background media or not
 *
 * @param {Object} background
 * @returns {Boolean}
 */
const hasBackground = (background) => {
  const {
    mediaType = IMAGE_BACKGROUND_TYPE,
    [mediaType]: media = {},
  } = background;
  return (
    (mediaType === IMAGE_BACKGROUND_TYPE ||
      mediaType === VIDEO_BACKGROUND_TYPE) &&
    media?.url
  );
};

/**
 * Get block classes for the feature.
 *
 * @param {Object} background
 * @param {Object} attributes
 */
function getBlockClasses(background, attributes) {
  const classes = [];

  if (
    hasBackground(background) ||
    (attributes?.isBlockEdit &&
      background?.image?.useFeaturedImage &&
      background?.mediaType !== VIDEO_BACKGROUND_TYPE)
  ) {
    classes.push("bb:has-background");
    classes.push(
      `bb:has-background--${background?.mediaType ?? IMAGE_BACKGROUND_TYPE}`
    );

    if (
      background?.settings?.animation?.type &&
      background?.settings?.animation?.type !== "none"
    ) {
      classes.push(
        `js-animation-${background?.settings?.animation?.type ?? ""}`
      );
    }
  }

  return classes.join(" ");
}

/**
 * Override the default edit UI to include new inspector controls for custom settings.
 *
 * @param {Function} BlockEdit Block edit component.
 * @return {Function} BlockEdit Modified block edit component.
 */
const withInspectorControls = createHigherOrderComponent((BlockEdit) => {
  return (props) => {
    if (!boldblocksHasSupport(props.name, featureName)) {
      return <BlockEdit {...props} />;
    }

    const { attributes, setAttributes, clientId, isSelected } = props;
    const {
      boldblocks = {},
      boldblocks: { background = {} } = {},
    } = attributes;
    const { mediaType = IMAGE_BACKGROUND_TYPE } = background;
    const { [mediaType]: media = {}, settings = {} } = background;
    const { animation = {} } = settings;
    const { type: animationType } = animation;

    const imageSizes = useSelect(
      (select) => select(blockEditorStore).getSettings().imageSizes,
      []
    );

    let sizeOptions = [];
    if (media?.useFeaturedImage) {
      sizeOptions = imageSizes?.length
        ? imageSizes.reduce((prev, item) => {
            return [...prev, { value: item?.slug, label: item?.name }];
          }, [])
        : [];
    } else if (media?.id && media?.sizes?.full) {
      sizeOptions = Object.keys(media.sizes).reduce((prev, item) => {
        const { width, height } = media.sizes[item];
        let label = item;
        if (width && height) {
          label = `${label}(${width}x${height})`;
        }
        prev.push({ value: item, label });

        return prev;
      }, []);
    }

    useEffect(() => {
      if ("bg-parallax" === animationType) {
        window.dispatchEvent(
          new CustomEvent("boldblocks.background.initParallax", {
            detail: {
              idSelector: `block-${clientId}`,
            },
          })
        );
      } else if ("bg-zoom" === animationType) {
        window.dispatchEvent(
          new CustomEvent("boldblocks.animate.settingsUpdated", {
            detail: {
              idSelector: `block-bg-${clientId}`,
            },
          })
        );
      }

      return () => {
        if ("bg-parallax" === animationType) {
          window.dispatchEvent(
            new CustomEvent("boldblocks.background.removeParallax", {
              detail: {
                idSelector: `block-${clientId}`,
              },
            })
          );
        } else if ("bg-zoom" === animationType) {
          window.dispatchEvent(
            new CustomEvent("boldblocks.animate.dispose", {
              detail: {
                idSelector: `block-bg-${clientId}`,
              },
            })
          );
        }
      };
    }, [animationType, clientId]);

    // Event handler for media change
    const handleBackgroundMediaChange = handleChangeSettingGroupField({
      setAttributes,
      attributes,
      fieldName: mediaType,
      groupName: "background",
    });

    const label = __("Background media", "content-blocks-builder");

    return (
      <>
        <BlockEdit {...props} />
        {isSelected && (
          <Fill name="cbb-panel-background">
            <ToolsPanelItem
              label={label}
              panelId="cbb-panel-background"
              hasValue={() =>
                hasBackground(background) ||
                (mediaType === IMAGE_BACKGROUND_TYPE && media?.useFeaturedImage)
              }
              onDeselect={() =>
                setAttributes({
                  boldblocks: { ...boldblocks, background: {} },
                })
              }
            >
              <BaseControl label={label} />
              <TabPanelStyled
                className="background-settings"
                initialTabName={mediaType}
                onSelect={(type) => {
                  if (mediaType !== type) {
                    handleChangeSettingGroupField({
                      setAttributes,
                      attributes,
                      fieldName: "mediaType",
                      groupName: "background",
                    })(type);
                  }
                }}
                tabs={MEDIA_TYPES}
              >
                {(tab) => <Slot name={`background-slot-${tab.name}`} />}
              </TabPanelStyled>
              <Fill name="background-slot-image">
                <PanelBodyInner>
                  <CustomMediaUpload
                    mediaType={IMAGE_BACKGROUND_TYPE}
                    media={media}
                    label={__("Set background image", "content-blocks-builder")}
                    inputLabel={__("Image URL", "content-blocks-builder")}
                    inputPlaceholder="https://example.com/bg.jpg"
                    changeMediaLabel={__(
                      "Change image",
                      "content-blocks-builder"
                    )}
                    removeMediaLabel={__(
                      "Remove image",
                      "content-blocks-builder"
                    )}
                    onMediaChange={(newMedia) =>
                      handleBackgroundMediaChange({
                        ...newMedia,
                        customAlt: media?.customAlt
                          ? media.customAlt
                          : newMedia?.alt,
                        useFeaturedImage: media?.useFeaturedImage,
                      })
                    }
                    renderPreview={(media) => (
                      <img
                        src={media?.url}
                        alt={__("Media preview", "content-blocks-builder")}
                      />
                    )}
                  />
                  <ToggleControl
                    label={__(
                      "Use the featured image",
                      "content-blocks-builder"
                    )}
                    checked={media?.useFeaturedImage ?? false}
                    onChange={(useFeaturedImage) =>
                      handleBackgroundMediaChange({
                        ...media,
                        useFeaturedImage,
                      })
                    }
                  />
                  {media?.useFeaturedImage && (
                    <ToggleControl
                      label={__("Link to post", "content-blocks-builder")}
                      checked={media?.linkToPost ?? false}
                      onChange={(linkToPost) =>
                        handleBackgroundMediaChange({
                          ...media,
                          linkToPost,
                        })
                      }
                    />
                  )}
                  {!!sizeOptions?.length && (
                    <SelectControl
                      label={__("Image size", "content-blocks-builder")}
                      value={media?.size ?? "full"}
                      onChange={(size) =>
                        handleBackgroundMediaChange({
                          ...media,
                          size,
                        })
                      }
                      options={sizeOptions}
                    />
                  )}
                </PanelBodyInner>
              </Fill>
              <Fill name="background-slot-video">
                <CustomMediaUpload
                  mediaType={VIDEO_BACKGROUND_TYPE}
                  media={media}
                  label={__("Set background video", "content-blocks-builder")}
                  inputLabel={__("Video URL", "content-blocks-builder")}
                  inputPlaceholder="https://example.com/bg.mp4"
                  changeMediaLabel={__(
                    "Change video",
                    "content-blocks-builder"
                  )}
                  removeMediaLabel={__(
                    "Remove video",
                    "content-blocks-builder"
                  )}
                  onMediaChange={handleBackgroundMediaChange}
                  renderPreview={(media) => (
                    <video src={media?.url} autoPlay={true} loop muted={true} />
                  )}
                />
              </Fill>
              <BackgroundSettings
                clientId={clientId}
                media={media}
                mediaType={mediaType}
                settings={settings}
                onBackgroundSettingsChange={handleChangeSettingGroupField({
                  setAttributes,
                  attributes,
                  fieldName: "settings",
                  groupName: "background",
                })}
                onBackgroundMediaChange={handleBackgroundMediaChange}
              />
            </ToolsPanelItem>
          </Fill>
        )}
      </>
    );
  };
}, "withInspectorControls");
addFilter(
  "editor.BlockEdit",
  `boldblocks/${featureName}/withInspectorControls`,
  withInspectorControls
);

/**
 * Override props assigned to save component to inject the CSS variables definition.
 *
 * @param {Object} props      Additional props applied to save element.
 * @param {Object} blockType  Block type.
 * @param {Object} attributes Block attributes.
 *
 * @return {Object} Filtered props applied to save element.
 */
export function addSaveProps(props, blockType, attributes) {
  if (!boldblocksHasSupport(blockType, featureName)) {
    return props;
  }

  const { boldblocks: { background = {} } = {} } = attributes;
  props.className = classnames(
    props.className,
    getBlockClasses(background, attributes)
  );

  return props;
}
// addFilter(
//   "blocks.getSaveContent.extraProps",
//   `boldblocks/${featureName}/addSaveProps`,
//   addSaveProps
// );

/**
 * Filters registered block settings to extend the block edit wrapper
 * to apply the desired styles and classnames properly.
 *
 * @param {Object} settings Original block settings.
 *
 * @return {Object}.Filtered block settings.
 */
addFilter(
  "blocks.registerBlockType",
  `boldblocks/${featureName}/addEditProps`,
  addEditProps(
    addSaveProps,
    (settings, { featureName }) => boldblocksHasSupport(settings, featureName),
    { featureName }
  )
);

/**
 * Build block's background markup
 *
 * @param {String} content
 * @param {Object} { name, clientId, attributes }
 */
const addBlockBackground = (content, props, isBlockEdit = false) => {
  const {
    name,
    attributes = {},
    supports: { BoldBlocksDeprecatedDynamicBGOverlay } = {},
  } = props;

  if (!boldblocksHasSupport(name, featureName)) {
    return content;
  }

  // Using new dynamic overlay, add deprecated check for legacy blocks.
  if (!isBlockEdit && !BoldBlocksDeprecatedDynamicBGOverlay) {
    return content;
  }

  // Get background
  const { boldblocks: { background = {} } = {} } = attributes;

  // Check the featured image
  if (isBlockEdit && hasFeaturedImageAsBackground(props)) {
    return (
      <FeaturedImageBackground
        context={props?.context}
        clientId={props.clientId}
        background={background}
        settings={background?.settings ?? {}}
        content={content}
        isBlockEdit={isBlockEdit}
      />
    );
  }

  // Bail if there is no selected media or don't use the featured image
  if (!hasBackground(background)) {
    return content;
  }

  const {
    mediaType = IMAGE_BACKGROUND_TYPE,
    [mediaType]: media = {},
    settings = {},
  } = background;

  let backgroundControl = null;

  if (mediaType === IMAGE_BACKGROUND_TYPE) {
    backgroundControl = (
      <BackgroundImage
        media={media}
        settings={settings}
        content={content}
        clientId={props.clientId}
        isBlockEdit={isBlockEdit}
      />
    );
  } else if (mediaType === VIDEO_BACKGROUND_TYPE) {
    const objectPosition =
      isObject(settings?.focalPoint) && !isEmpty(settings?.focalPoint)
        ? `${parseFloat(settings?.focalPoint?.x ?? 0.5).toFixed(2) * 100}% ${
            parseFloat(settings?.focalPoint?.y ?? 0.5).toFixed(2) * 100
          }%`
        : "50% 50%";

    let objectFit = settings?.objectFit ? settings?.objectFit : "cover";
    if (objectFit === "cover") {
      objectFit = null;
    }

    backgroundControl = (
      <>
        <div
          className={classnames(
            "bb:block-background bb:block-background--video"
          )}
        >
          <video
            src={media?.url}
            autoplay="autoplay"
            muted
            loop
            playsinline
            preload="auto"
            style={{ objectPosition, objectFit }}
          />
        </div>
        {content}
      </>
    );
  }

  return backgroundControl;
};
addFilter(
  "boldblocks.edit.blockInner",
  `boldblocks/${featureName}/addBlockBackground`,
  addBlockBackground
);
addFilter(
  "boldblocks.save.blockInner",
  `boldblocks/${featureName}/addBlockBackground`,
  addBlockBackground
);

const hasFeaturedImageAsBackground = (props) =>
  props?.attributes?.boldblocks?.background?.mediaType !== "video" &&
  props?.attributes?.boldblocks?.background?.image?.useFeaturedImage &&
  props?.context?.postId &&
  props?.context?.postType;

const BackgroundImage = ({
  media,
  settings,
  content,
  clientId,
  isBlockEdit,
}) => {
  const backgroundPosition =
    isObject(settings?.focalPoint) && !isEmpty(settings?.focalPoint)
      ? `${parseFloat(settings?.focalPoint?.x ?? 0.5).toFixed(2) * 100}% ${
          parseFloat(settings?.focalPoint?.y ?? 0.5).toFixed(2) * 100
        }%`
      : "50% 50%";

  const { isImgElement } = settings;

  let backgroundStyle = {};
  let dataset = {};
  if (isBlockEdit) {
    dataset = { id: `block-bg-${clientId}` };
  }
  let imgElement = null;
  let imgUrl = media?.url ?? media?.source_url;
  if (media?.size && media?.sizes) {
    const sizeObject = media.sizes[media.size];
    if (sizeObject?.url || sizeObject?.source_url) {
      imgUrl = sizeObject?.url ?? sizeObject?.source_url;
    }
  }
  if (isImgElement) {
    const imgStyle = {
      width: "100%",
      height: "100%",
      objectPosition: backgroundPosition,
      objectFit: settings?.objectFit ? settings.objectFit : "cover",
    };
    imgElement = (
      <img
        src={imgUrl}
        alt={media?.customAlt ?? media?.alt_text ?? ""}
        style={imgStyle}
        className={classnames("bb:block-background--img", {
          [`wp-image-${media?.id}`]: media?.id,
        })}
      />
    );
  } else {
    backgroundStyle = {
      backgroundImage: `url("${imgUrl}")`,
      backgroundPosition,
      backgroundAttachment: settings?.isFixed ? "fixed" : undefined,
      backgroundRepeat: settings?.repeat ?? "no-repeat",
      backgroundSize: settings?.size ?? "cover",
    };
  }

  const animationType = settings?.animation?.type;
  const animationSettings = settings?.animation?.settings ?? {};
  if (animationType) {
    if (
      animationType === "bg-scroll-horizontal" ||
      animationType === "bg-scroll-vertical"
    ) {
      if (!isImgElement) {
        if (hasAnimationSettingValue("scroll", "duration", animationSettings)) {
          backgroundStyle.animationDuration = `${getAnimationSettingValue(
            "scroll",
            "duration",
            animationSettings
          )}s`;
        }

        // Make the background repeat based on the animation
        if (!settings?.repeat || settings?.repeat !== "repeat") {
          backgroundStyle.backgroundRepeat =
            animationType === "bg-scroll-horizontal" ? "repeat-x" : "repeat-y";
        }
      }
    } else if (animationType === "bg-parallax") {
      // Remove fixed background style for parallax effect
      backgroundStyle.backgroundAttachment = undefined;

      dataset["data-speed"] = hasAnimationSettingValue(
        "parallax",
        "speed",
        animationSettings
      )
        ? getAnimationSettingValue("parallax", "speed", animationSettings)
        : 0.5;

      if (hasAnimationSettingValue("parallax", "opacity", animationSettings)) {
        dataset["data-opacity"] = true;
      }

      if (
        hasAnimationSettingValue(
          "parallax",
          "disableParallax",
          animationSettings
        )
      ) {
        dataset["data-disable-parallax"] = true;

        dataset["data-disable-parallax-breakpoint"] = hasAnimationSettingValue(
          "parallax",
          "disableParallaxBreakpoint",
          animationSettings
        )
          ? getAnimationSettingValue(
              "parallax",
              "disableParallaxBreakpoint",
              animationSettings
            )
          : 767;
      }
    } else if (animationType === "bg-zoom") {
      if (animationSettings?.zoom?.initialScale) {
        backgroundStyle["--cbb--zoom-initial-scale"] =
          animationSettings.zoom.initialScale;
      }
      if (animationSettings?.zoom?.scale) {
        backgroundStyle["--cbb--zoom-scale"] = animationSettings.zoom.scale;
      }
      if (animationSettings?.zoom?.duration) {
        backgroundStyle[
          "--cbb--zoom-duration"
        ] = `${animationSettings.zoom.duration}s`;
      }
      if (animationSettings?.zoom?.timingFunction) {
        backgroundStyle["--cbb--zoom-timing-function"] =
          animationSettings.zoom.timingFunction;
      }

      if (animationSettings?.zoom?.event === "reveal") {
        dataset["data-reveal-animation"] = JSON.stringify({
          animationName: "bgZoom",
        });

        dataset["data-animate-multiple-times"] =
          animationSettings?.zoom?.multipleTimes;
      }
    }
  }

  return (
    <>
      <div
        className={classnames(
          "bb:block-background bb:block-background--image",
          {
            [animationType]: animationType && animationType !== "none",
            "is-reverse":
              (animationType === "bg-scroll-horizontal" ||
                animationType === "bg-scroll-vertical") &&
              getAnimationSettingValue(
                "scroll",
                "reverseDirection",
                animationSettings
              ),
            [`bg-zoom-${animationSettings?.zoom?.event ?? "hover"}`]:
              animationType === "bg-zoom",
          }
        )}
        style={backgroundStyle}
        {...dataset}
      >
        {imgElement}
      </div>
      {content}
    </>
  );
};

const FeaturedImageBackground = ({
  context,
  clientId,
  settings,
  content,
  background,
  isBlockEdit,
}) => {
  const {
    mediaType = IMAGE_BACKGROUND_TYPE,
    [mediaType]: media = {},
  } = background;

  const featureImage = useFeaturedMedia(context);
  if (featureImage?.id && featureImage?.url) {
    requestAnimationFrame(() => {
      if (settings?.animation?.type === "bg-parallax") {
        window.dispatchEvent(
          new CustomEvent("boldblocks.background.updateParallax", {
            detail: {
              idSelector: `block-${clientId}`,
              animationType: settings?.animation?.type,
            },
          })
        );
      } else if (settings?.animation?.type === "bg-zoom") {
        window.dispatchEvent(
          new CustomEvent("boldblocks.animate.settingsUpdated", {
            detail: {
              idSelector: `block-bg-${clientId}`,
            },
          })
        );
      }
    });

    return (
      <BackgroundImage
        media={{ ...featureImage, size: media?.size }}
        settings={settings}
        content={content}
        clientId={clientId}
        isBlockEdit={isBlockEdit}
      />
    );
  } else if (hasBackground(background)) {
    if (mediaType === IMAGE_BACKGROUND_TYPE) {
      return (
        <BackgroundImage
          media={media}
          settings={settings}
          content={content}
          clientId={clientId}
          isBlockEdit={isBlockEdit}
        />
      );
    }
  }

  return null;
};
