/**
 * External dependencies
 */

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { addFilter } from "@wordpress/hooks";
import { createHigherOrderComponent } from "@wordpress/compose";
import {
  InspectorControls,
  store as blockEditorStore,
} from "@wordpress/block-editor";
import { useSelect, useDispatch } from "@wordpress/data";
import { PanelBody } from "@wordpress/components";
import {
  serialize,
  createBlocksFromInnerBlocksTemplate,
} from "@wordpress/blocks";
import apiFetch from "@wordpress/api-fetch";
import { useState } from "@wordpress/element";
import { store as noticesStore } from "@wordpress/notices";

/**
 * Internal dependencies
 */
import BlockAdditionalInfo from "../../components/block-additional-info";
import { boldblocksHasSupport, CoreBoldBlocks } from "../../utils";

/**
 * Define feature name
 */
const featureName = "blockInfo";

const canEdit = (blockName) => !CoreBoldBlocks.includes(blockName);

const canSyncContent = (blockName) => {
  return boldblocksHasSupport(blockName, "syncContent") && canEdit(blockName);
};

/**
 * Override the default edit UI to include new inspector controls for custom settings.
 *
 * @param {Function} BlockEdit Block edit component.
 * @return {Function} BlockEdit Modified block edit component.
 */
const withInspectorControls = createHigherOrderComponent((BlockEdit) => {
  return (props) => {
    const blockInfo = boldblocksHasSupport(props.name, featureName);
    if (!blockInfo) {
      return <BlockEdit {...props} />;
    }

    const { getBlocks } = useSelect((select) => select(blockEditorStore), []);
    const { replaceInnerBlocks } = useDispatch(blockEditorStore);
    const { createSuccessNotice } = useDispatch(noticesStore);
    const [isUpdating, setIsUpdating] = useState(false);

    const { isSelected, clientId, name } = props;

    return (
      <>
        <BlockEdit {...props} />
        {isSelected && (
          <>
            <InspectorControls>
              <PanelBody
                title={__(
                  "Block additional information",
                  "content-blocks-builder"
                )}
                initialOpen={false}
              >
                <BlockAdditionalInfo
                  blockInfo={blockInfo}
                  isUpdating={isUpdating}
                  canSyncContent={canSyncContent(name)}
                  canEdit={canEdit(name)}
                  onResetContent={(template) => {
                    replaceInnerBlocks(
                      clientId,
                      createBlocksFromInnerBlocksTemplate(template)
                    );
                  }}
                  onPushContent={(id) => {
                    setIsUpdating(true);
                    const blocks = getBlocks(clientId);
                    apiFetch({
                      path: `wp/v2/boldblocks-blocks/${id}`,
                      method: "POST",
                      data: {
                        content: serialize(blocks),
                        meta: {
                          boldblocks_block_blocks: JSON.stringify(blocks),
                        },
                      },
                    })
                      .then(() => {
                        createSuccessNotice(
                          __(
                            "The default block content has been updated successfully!",
                            "content-blocks-builder"
                          ),
                          {
                            type: "snackbar",
                          }
                        );
                      })
                      .finally(() => setIsUpdating(false));
                  }}
                />
              </PanelBody>
            </InspectorControls>
          </>
        )}
      </>
    );
  };
}, "withInspectorControls");
addFilter(
  "editor.BlockEdit",
  `boldblocks/${featureName}/withInspectorControls`,
  withInspectorControls
);
