/**
 * External dependencies
 */
import classnames from "classnames";
import { isObject, isString, isEmpty, isUndefined } from "lodash";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { addFilter } from "@wordpress/hooks";
import { createHigherOrderComponent } from "@wordpress/compose";
import { InspectorControls } from "@wordpress/block-editor";
import {
  PanelBody,
  ToggleControl,
  SelectControl,
  ExternalLink,
} from "@wordpress/components";
import { hasBlockSupport } from "@wordpress/blocks";

/**
 * Internal dependencies
 */
import { PanelBodyInner, ToggleGroupCustomControl } from "sdk/components";
import {
  getBreakpointType,
  getAllBreakpoints,
  handleChangeResponsiveSettingGroupField,
  getResponsiveSettingGroupFieldValue,
  handleChangeSettingGroupField,
  getSettingGroupFieldValue,
  isUnsetValue,
  addEditProps,
  usePropsStyle,
} from "sdk/utils";
import { boldblocksHasSupport, isPremium } from "../../utils";
import { LabelHelpControl } from "../../components/label-help-control";

/**
 * Define feature name
 */
const featureName = "sticky";

/**
 * Get styles for the feature.
 *
 * @param {Object} sticky
 */
function getFeatureStyle(sticky) {
  let style = {};
  if (!sticky?.enable) {
    return style;
  }

  const offset = sticky?.offset;

  if (isObject(offset) && !isEmpty(offset)) {
    Object.keys(offset).forEach((breakpoint) => {
      const { value, inherit } = offset[breakpoint];
      if (isUndefined(value)) {
        if (inherit && isString(inherit)) {
          const { value: { value: inheritValue } = {} } = offset[inherit] ?? {};
          if (!isUnsetValue(inheritValue)) {
            style = {
              ...style,
              [`--bb--sticky-offset--${breakpoint}`]: `var(--bb--sticky-offset--${inherit})`,
            };
          }
        }
      } else {
        if (value) {
          const { value: cssValue } = value ?? {};
          if (!isUnsetValue(cssValue)) {
            style = {
              ...style,
              [`--bb--sticky-offset--${breakpoint}`]: cssValue + "",
            };
          }
        }
      }
    });
  }

  return style;
}

/**
 * Override the default edit UI to include new inspector controls for custom settings.
 *
 * @param {Function} BlockEdit Block edit component.
 * @return {Function} BlockEdit Modified block edit component.
 */
const withInspectorControls = createHigherOrderComponent((BlockEdit) => {
  return (props) => {
    if (!boldblocksHasSupport(props.name, featureName)) {
      return <BlockEdit {...props} />;
    }

    const { attributes, setAttributes, isSelected } = props;

    // Get all breakpoints
    const allBreakpoints = getAllBreakpoints();

    // Get current breakpoint
    const breakpoint = getBreakpointType();

    // Get group field value
    const getStickyGroupFieldValue = ({ fieldName, defaultValue }) =>
      getSettingGroupFieldValue({
        fieldName,
        defaultValue,
        attributes,
        groupName: "sticky",
      });

    // Handle group field change
    const handleStickyGroupFieldChange = (fieldName) =>
      handleChangeSettingGroupField({
        fieldName,
        groupName: "sticky",
        setAttributes,
        attributes,
      });

    // Enable
    const enableSticky = getStickyGroupFieldValue({
      fieldName: "enable",
      defaultValue: false,
    });

    const isFixed = getStickyGroupFieldValue({
      fieldName: "isFixed",
      defaultValue: false,
    });

    let scrollingUpLabel = __(
      "Stick to the top when scrolling up",
      "content-blocks-builder"
    );

    if (!isPremium) {
      scrollingUpLabel = `${scrollingUpLabel} (premium)`;
    }

    const isStickOnScrollingUp = getStickyGroupFieldValue({
      fieldName: "isStickOnScrollingUp",
      defaultValue: false,
    });

    return (
      <>
        <BlockEdit {...props} />
        {isSelected && (
          <>
            <InspectorControls>
              <PanelBody
                title={__("Sticky block", "content-blocks-builder")}
                initialOpen={enableSticky}
                className="boldblocks-panel-settings is-sticky-settings"
              >
                <PanelBodyInner>
                  <ToggleControl
                    label={__("Enable sticky", "content-blocks-builder")}
                    checked={enableSticky}
                    onChange={handleStickyGroupFieldChange("enable")}
                    help={__(
                      "Turn this block into a sticky block.",
                      "content-blocks-builder"
                    )}
                  />
                  {enableSticky && (
                    <>
                      <ToggleControl
                        label={__(
                          "Always stick to the viewport",
                          "content-blocks-builder"
                        )}
                        checked={isFixed}
                        onChange={handleStickyGroupFieldChange("isFixed")}
                        help={
                          <>
                            {__(
                              "Use position fixed instead of position sticky. Learn more about ",
                              "content-blocks-builder"
                            )}
                            <ExternalLink href="https://developer.mozilla.org/en-US/docs/Web/CSS/position">
                              {__(
                                "sticky and fixed.",
                                "content-blocks-builder"
                              )}
                            </ExternalLink>
                          </>
                        }
                        disabled={isStickOnScrollingUp}
                      />
                      <ToggleControl
                        label={scrollingUpLabel}
                        checked={isStickOnScrollingUp}
                        onChange={handleStickyGroupFieldChange(
                          "isStickOnScrollingUp"
                        )}
                        disabled={!isPremium}
                      />
                      {!isStickOnScrollingUp && (
                        <>
                          <SelectControl
                            label={__("Stick to", "content-blocks-builder")}
                            value={getStickyGroupFieldValue({
                              fieldName: "stickTo",
                              defaultValue: "top",
                            })}
                            options={[
                              {
                                value: "top",
                                label: __("Top", "content-blocks-builder"),
                              },
                              {
                                value: "bottom",
                                label: __("Bottom", "content-blocks-builder"),
                              },
                            ]}
                            onChange={handleStickyGroupFieldChange("stickTo")}
                            help={__(
                              "Only top/bottom are supported here, please set left, right in the Custom CSS setting if necessary."
                            )}
                            disabled={isStickOnScrollingUp}
                          />
                          <ToggleGroupCustomControl
                            name="offset"
                            label={__("Offset value", "content-blocks-builder")}
                            value={getResponsiveSettingGroupFieldValue({
                              fieldName: "offset",
                              groupName: "sticky",
                              attributes,
                              breakpoint,
                            })}
                            onChange={handleChangeResponsiveSettingGroupField({
                              fieldName: "offset",
                              groupName: "sticky",
                              setAttributes,
                              attributes,
                              breakpoint,
                              allBreakpoints,
                            })}
                            options={[
                              {
                                value: "0",
                                label: "0",
                              },
                              {
                                value: "auto",
                                label: __("Auto", "content-blocks-builder"),
                              },
                              {
                                value: "custom",
                                label: __(
                                  "Custom value",
                                  "content-blocks-builder"
                                ),
                              },
                            ]}
                            isResponsive={true}
                          />
                        </>
                      )}
                      {!isFixed && !isStickOnScrollingUp && (
                        <LabelHelpControl
                          label={__(
                            "Troubleshooting",
                            "content-blocks-builder"
                          )}
                          helpControls={
                            <>
                              <ul
                                style={{
                                  listStyle: "disc",
                                  listStylePosition: "outside",
                                  paddingLeft: "15px",
                                }}
                              >
                                <li>
                                  {__(
                                    'You must set a value other than "auto" for at least one of following properties: top, right, bottom, left.'
                                  )}
                                </li>
                                <li>
                                  {__(
                                    'Sticky will most probably not work if any parent of the element has a value of overflow other than "visible".'
                                  )}
                                </li>
                                <li>
                                  {__(
                                    "Sticky will may not work correctly if any parent of the element has a set height."
                                  )}
                                </li>
                                <li>
                                  <ExternalLink href="https://developer.mozilla.org/en-US/docs/Web/CSS/position#sticky_positioning">
                                    {__(
                                      "Learn about position sticky",
                                      "content-blocks-builder"
                                    )}
                                  </ExternalLink>
                                </li>
                              </ul>
                            </>
                          }
                        />
                      )}
                    </>
                  )}
                </PanelBodyInner>
              </PanelBody>
            </InspectorControls>
          </>
        )}
      </>
    );
  };
}, "withInspectorControls");
addFilter(
  "editor.BlockEdit",
  `boldblocks/${featureName}/withInspectorControls`,
  withInspectorControls
);

/**
 * Override props assigned to save component to inject the CSS variables definition.
 *
 * @param {Object} props      Additional props applied to save element.
 * @param {Object} blockType  Block type.
 * @param {Object} attributes Block attributes.
 *
 * @return {Object} Filtered props applied to save element.
 */
export function addSaveProps(props, blockType, attributes) {
  if (!boldblocksHasSupport(blockType, featureName)) {
    return props;
  }

  const { boldblocks: { sticky = {} } = {} } = attributes;

  if (sticky?.enable) {
    const stickTo = sticky?.stickTo ?? "top";

    props.className = classnames(props.className, "is-sticky-block", {
      [`is-stick-to-${stickTo}`]: stickTo,
      "is-fixed": sticky?.isFixed && !sticky?.isStickOnScrollingUp,
      "is-sticky-on-scrollup": sticky?.isStickOnScrollingUp,
    });
  }

  const isDeprecatedDynamicStyle = hasBlockSupport(
    blockType,
    "CBBDeprecatedDynamicStyleV1"
  );

  let featureStyles = {};
  if (attributes?.isBlockEdit) {
    featureStyles = usePropsStyle({
      value: sticky,
      getStyle: (value) => () => getFeatureStyle(value),
    });
  } else {
    if (isDeprecatedDynamicStyle) {
      featureStyles = getFeatureStyle(sticky);
    }
  }

  props.style = {
    ...props.style,
    ...featureStyles,
  };

  return props;
}
addFilter(
  "blocks.getSaveContent.extraProps",
  `boldblocks/${featureName}/addSaveProps`,
  addSaveProps
);

/**
 * Filters registered block settings to extend the block edit wrapper
 * to apply the desired styles and classnames properly.
 *
 * @param {Object} settings Original block settings.
 *
 * @return {Object}.Filtered block settings.
 */
addFilter(
  "blocks.registerBlockType",
  `boldblocks/${featureName}/addEditProps`,
  addEditProps(
    addSaveProps,
    (settings, { featureName }) => boldblocksHasSupport(settings, featureName),
    { featureName }
  )
);
