/**
 * External dependencies
 */
import { isUndefined } from "lodash";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { useSelect } from "@wordpress/data";
import { useEntityProp } from "@wordpress/core-data";
import { registerPlugin } from "@wordpress/plugins";
import {
  PluginSidebar,
  PluginSidebarMoreMenuItem,
  PluginDocumentSettingPanel,
} from "@wordpress/edit-post";
import {
  PanelBody,
  CheckboxControl,
  Spinner,
  BaseControl,
  ExternalLink,
  TextareaControl,
  ToggleControl,
} from "@wordpress/components";
import { store as editorStore } from "@wordpress/editor";
import { useState, useEffect } from "@wordpress/element";
import apiFetch from "@wordpress/api-fetch";
import { addQueryArgs } from "@wordpress/url";
import { applyFilters } from "@wordpress/hooks";

/**
 * Internal dependencies
 */
import { userCanUpdateSettings } from "../utils/role-and-cap";

/**
 * Register the custom pattern meta.
 */
registerPlugin("boldblocks-pattern-meta", {
  render: () => {
    const { getCurrentPostType } = useSelect(
      (select) => select(editorStore),
      []
    );
    const [isLoading, setIsLoading] = useState(true);
    const [allCategories, setAllCategories] = useState([]);

    // Only display block settings for boldblocks_pattern content type
    if (getCurrentPostType() !== "boldblocks_pattern") {
      return null;
    }

    // Get pattern categories.
    useEffect(() => {
      apiFetch({ path: "boldblocks/v1/getPatternCategories" }).then((res) => {
        setAllCategories(res);
        setIsLoading(false);
      });
    }, []);

    // Get meta, setMeta
    const [meta, setMeta] = useEntityProp(
      "postType",
      "boldblocks_pattern",
      "meta"
    );

    const updateMeta = (name) => (newValue) => {
      setMeta({ ...meta, [name]: newValue });
    };

    const {
      boldblocks_pattern_categories: patternCategories = [],
      boldblocks_pattern_description: patternDescription = "",
      boldblocks_pattern_disabled_inserter: disabledInserter = false,
    } = meta;

    const pluginTitle = __("Pattern settings", "content-blocks-builder");

    const PluginControls = (
      <>
        <TextareaControl
          label={__("Pattern description", "content-blocks-builder")}
          value={patternDescription}
          onChange={updateMeta("boldblocks_pattern_description")}
          rows={4}
        />
        <ToggleControl
          checked={disabledInserter}
          label={__(
            "Don't show this pattern in the inserter",
            "content-blocks-builder"
          )}
          onChange={updateMeta("boldblocks_pattern_disabled_inserter")}
        />
        <BaseControl
          label={__("Select pattern categories", "content-blocks-builder")}
        />
        {!!allCategories.length ? (
          <>
            {allCategories
              .filter(({ name }) => name !== "boldblocks")
              .map(({ name, label }) => (
                <CheckboxControl
                  key={name}
                  label={label}
                  checked={patternCategories.find(
                    ({ name: itemName }) => itemName === name
                  )}
                  onChange={(checked) => {
                    let newCategories = [];
                    if (checked) {
                      newCategories = [...patternCategories, { name, label }];
                    } else {
                      newCategories = patternCategories.filter(
                        ({ name: itemName }) => itemName !== name
                      );
                    }
                    updateMeta("boldblocks_pattern_categories")(newCategories);
                  }}
                />
              ))}
            {userCanUpdateSettings && (
              <p>
                <ExternalLink
                  href={addQueryArgs(
                    "edit.php?post_type=boldblocks_block&page=cbb-settings&tab=others"
                  )}
                >
                  {__("Manage custom categories", "content-blocks-builder")}
                </ExternalLink>
              </p>
            )}
          </>
        ) : (
          <>{isLoading && <Spinner />}</>
        )}
        {applyFilters("boldblocks.patternSettings.additionalSettings", null, {
          meta,
          setMeta,
        })}
      </>
    );

    return (
      <>
        <PluginDocumentSettingPanel
          name="content-block-settings"
          title={pluginTitle}
          initialOpen={true}
        >
          {PluginControls}
        </PluginDocumentSettingPanel>
        <PluginSidebarMoreMenuItem target="pattern-settings">
          {pluginTitle}
        </PluginSidebarMoreMenuItem>
        <PluginSidebar name="pattern-settings" title={pluginTitle}>
          <PanelBody>{PluginControls}</PanelBody>
        </PluginSidebar>
      </>
    );
  },
});
