/**
 * External dependencies
 */

/**
 * WordPress dependencies
 */
import apiFetch from "@wordpress/api-fetch";
import { addQueryArgs } from "@wordpress/url";

/**
 * Internal dependencies
 */

/**
 * Constants
 */

/**
 * Initial state
 */
const initialState = {
  patternInserter: {
    status: false,
    modalState: {},
    patterns: [],
    patternKeywords: [],
    missingBlocks: {},
    missingBlocksStatuses: {},
    plugins: [],
  },
};

export const patternInserterSelectors = {
  getPatternInserterModalStatus(state) {
    return state.patternInserter.status;
  },
  getPatterns(state) {
    return state.patternInserter.patterns;
  },
  getPatternsModalState(state) {
    return state.patternInserter.modalState;
  },
  getPatternKeywords(state) {
    return state.patternInserter.patternKeywords;
  },
  getMissingBlock(state, filterValue) {
    return state.patternInserter.missingBlocks[filterValue] ?? false;
  },
  getMissingBlockStatus(state, id) {
    return state.patternInserter.missingBlocksStatuses[id] ?? false;
  },
  getPlugins(state) {
    return state.patternInserter.plugins;
  },
};

export const patternInserterResolvers = {
  getPatterns() {
    return async ({ dispatch }) => {
      const patterns = await apiFetch({ path: "boldblocks/v1/getPatterns" });

      if (patterns && patterns.length) {
        dispatch({
          type: "SET_PATTERNS",
          payload: patterns,
        });
      }

      return patterns;
    };
  },
  getPatternKeywords() {
    return async ({ dispatch }) => {
      const keywords = await apiFetch({
        path: "boldblocks/v1/getPatternKeywords",
      });

      if (keywords && keywords.length) {
        dispatch({
          type: "SET_PATTERN_KEYWORDS",
          payload: keywords,
        });
      }

      return keywords;
    };
  },
  getPlugins() {
    return async ({ dispatch }) => {
      let plugins = await apiFetch({
        path: `wp/v2/plugins`,
      });

      plugins = plugins.map((item) => {
        const { plugin } = item;
        const slug = plugin.split("/")[0];

        return { ...item, slug };
      });

      dispatch({
        type: "SET_PLUGINS",
        payload: plugins,
      });
    };
  },
};

export const patternInserterActions = {
  setPatternInserterModalStatus(status) {
    return {
      type: "SET_PATTERN_INSERTER_MODAL_STATUS",
      payload: status,
    };
  },
  loadFullPatterns(patternIds) {
    return async ({ dispatch }) => {
      if (!patternIds.length) {
        return;
      }

      const patterns = await apiFetch({
        path: addQueryArgs(`boldblocks/v1/getFullPatternData`, {
          patternIds: patternIds.join(","),
        }),
      });

      if (patterns && patterns.length) {
        dispatch({
          type: "UPDATE_PATTERNS",
          payload: patterns,
        });
      }

      return patterns;
    };
  },
  setPatternsModalState(modalState) {
    return {
      type: "UPDATE_PATTERN_INSERTER_MODAL_STATE",
      payload: modalState,
    };
  },
  setMissingBlockStatus(id) {
    return {
      type: "SET_MISSING_BLOCK_STATUS",
      payload: id,
    };
  },
  loadMissingBlock(filterValue) {
    return async ({ select, dispatch }) => {
      let missingBlock = select.getMissingBlock(filterValue);
      if (missingBlock === false) {
        const blocks = await apiFetch({
          path: `wp/v2/block-directory/search?term=${filterValue}`,
        });

        missingBlock = blocks[0] ?? {};

        dispatch({
          type: "SET_MISSING_BLOCK",
          payload: {
            [filterValue]: missingBlock,
          },
        });
      }

      return missingBlock;
    };
  },
};

export const patternInserterReducer = (
  state = initialState.patternInserter,
  action
) => {
  switch (action.type) {
    case "SET_PATTERN_INSERTER_MODAL_STATUS":
      return {
        ...state,
        status: action.payload,
      };

    case "SET_PATTERNS":
      return {
        ...state,
        patterns: [...action.payload],
      };

    case "UPDATE_PATTERNS":
      const patternIds = action.payload.map(({ id }) => id);
      const patterns = state.patterns.map((pattern) => {
        if (patternIds.includes(pattern.id)) {
          const patternFullData = action.payload.find(
            ({ id }) => id === pattern.id
          );
          if (patternFullData) {
            return patternFullData;
          }
        }

        return pattern;
      });

      return {
        ...state,
        patterns,
      };

    case "UPDATE_PATTERN_INSERTER_MODAL_STATE": {
      return {
        ...state,
        modalState: { ...action.payload },
      };
    }

    case "SET_PATTERN_KEYWORDS":
      return {
        ...state,
        patternKeywords: [...action.payload],
      };

    case "SET_MISSING_BLOCK":
      return {
        ...state,
        missingBlocks: {
          ...state.missingBlocks,
          ...action.payload,
        },
      };

    case "SET_MISSING_BLOCK_STATUS":
      return {
        ...state,
        missingBlocksStatuses: {
          ...state.missingBlocksStatuses,
          [action.payload]: true,
        },
      };

    case "SET_PLUGINS":
      return {
        ...state,
        plugins: [...action.payload],
      };
  }

  return state;
};
