/**
 * External dependencies
 */
import { isObject, isArray, map, isUndefined, isEmpty, pick } from "lodash";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { useSelect } from "@wordpress/data";
import { useEntityProp } from "@wordpress/core-data";
import { registerPlugin } from "@wordpress/plugins";
import {
  PluginSidebar,
  PluginSidebarMoreMenuItem,
  PluginDocumentSettingPanel,
} from "@wordpress/edit-post";
import {
  PanelBody,
  TextareaControl,
  ToggleControl,
} from "@wordpress/components";
import { store as editorStore } from "@wordpress/editor";
import { useEffect, useMemo, useRef } from "@wordpress/element";
import apiFetch from "@wordpress/api-fetch";
import {
  rawHandler,
  getBlockTypes,
  unregisterBlockVariation,
} from "@wordpress/blocks";
import { applyFilters } from "@wordpress/hooks";
import { store as noticesStore } from "@wordpress/notices";
import { useDispatch } from "@wordpress/data";

/**
 * Internal dependencies
 */
import { log } from "sdk/utils";
import { NestedPanelBody } from "sdk/components";
import { BlockIcon } from "../components/block-icon";
import { buildVariationData } from "./utils";
import {
  isPremium,
  getDependentBlockTypes,
  userCanManagePlugins,
  getEditPostURL,
  hasMissingBlocks,
  getNoticeActions,
  getPostBlocksData,
  refineCustomCSS,
} from "../utils";
import { store as boldblocksDataStore } from "../store";
import { updateDynamicStyle } from "../utils/editor";

/**
 * Update blocks metadata
 */
apiFetch.use((options, next) => {
  const { method, path } = options;
  if (method === "PUT" && path.indexOf("/wp/v2/boldblocks-variations") === 0) {
    const { data = {}, data: { content, meta = {} } = {} } = options;
    const { changedFields = [] } = meta;
    if (!content && !changedFields?.length) {
      return next(options);
    }

    let changedMeta = pick(meta, changedFields);

    if (content) {
      const postData = wp.data
        .select("core")
        .getEntityRecord("postType", "boldblocks_variation", data?.id);

      if (postData) {
        const {
          title: { raw: title },
          meta: {
            boldblocks_variation_data: storedVariationData,
            boldblocks_variation_block_name: blockName,
            boldblocks_variation_name: variationName,
          } = {},
        } = postData;

        const blocks = getPostBlocksData(rawHandler({ HTML: content }));

        const variationData = buildVariationData({
          blockName,
          variationName,
          title,
          block: blocks[0] ?? {},
        });

        const boldblocks_variation_data = JSON.stringify(variationData);
        if (boldblocks_variation_data !== storedVariationData) {
          changedMeta = {
            ...changedMeta,
            boldblocks_variation_data,
            boldblocks_variation_dependent_blocks:
              getDependentBlockTypes(blocks ?? []) ?? [],
          };
        }
      }
    }

    if (!isEmpty(changedMeta)) {
      log("Update variations metadata");
      log(changedMeta);
      options = { ...options, data: { ...data, meta: changedMeta } };
    }
  }
  return next(options);
});

/**
 * Register the custom variation meta.
 */
registerPlugin("boldblocks-variation-meta", {
  render: () => {
    const { getCurrentPostType } = useSelect(
      (select) => select(editorStore),
      []
    );

    // Only display block settings for boldblocks_variation content type
    if (getCurrentPostType() !== "boldblocks_variation") {
      return null;
    }

    // Get meta, setMeta
    const [meta, setMeta] = useEntityProp(
      "postType",
      "boldblocks_variation",
      "meta"
    );

    const changedMetaRef = useRef([]);

    const updateMeta = (name) => (newValue) => {
      if (!changedMetaRef.current.includes(name)) {
        changedMetaRef.current.push(name);
      }

      setMeta({
        ...meta,
        [name]: newValue,
        changedFields: changedMetaRef.current,
      });
    };

    const {
      boldblocks_variation_block_name: blockName,
      boldblocks_variation_icon: blockIcon = "",
      boldblocks_variation_description: description = "",
      boldblocks_variation_is_default: isDefault = false,
      boldblocks_variation_is_transformable: isTransformable = false,
      boldblocks_variation_hide_from_inserter: hideFromInserter = false,
      boldblocks_variation_custom_style: customStyle = "",
      boldblocks_variation_enable_style: enableStyle = false,
    } = meta;

    const pluginTitle = __("Variation settings", "content-blocks-builder");

    const PluginControls = (
      <>
        <BlockIcon
          label={__("Variation icon:", "content-blocks-builder")}
          value={blockIcon}
          onChange={updateMeta("boldblocks_variation_icon")}
        />
        <TextareaControl
          label={__("Variation description:", "content-blocks-builder")}
          value={description}
          onChange={updateMeta("boldblocks_variation_description")}
          rows={4}
        />
        <ToggleControl
          label={__("Is default variation?", "content-blocks-builder")}
          checked={isDefault}
          onChange={updateMeta("boldblocks_variation_is_default")}
        />
        <ToggleControl
          label={__("Is transformable?", "content-blocks-builder")}
          checked={isTransformable}
          onChange={updateMeta("boldblocks_variation_is_transformable")}
          help={__(
            "By enabling this setting, some 'content' attributes will not be included in this variation. For example: id, content, text, title. And it will not effect to inner bocks."
          )}
        />
        <ToggleControl
          label={__("Hide from the inserter?", "content-blocks-builder")}
          checked={hideFromInserter}
          onChange={updateMeta("boldblocks_variation_hide_from_inserter")}
          help={__(
            "Hide this variation from the inserter. However, it will still be shown in the variation picker popup.",
            "content-blocks-builder"
          )}
        />
        <ToggleControl
          label={__(
            "Register a block style for this variation",
            "content-blocks-builder"
          )}
          checked={enableStyle}
          onChange={updateMeta("boldblocks_variation_enable_style")}
          help={__(
            "By enabling this setting, a custom block style will be generated. It's very helpful if the variation has some custom CSS."
          )}
        />
        {applyFilters("boldblocks.variationSettings.additionalSettings", null, {
          meta,
          setMeta: updateMeta,
        })}
      </>
    );

    const manageStylesTitle = __(
      "Manage custom style",
      "content-blocks-builder"
    );

    const disabledStylesTitle = __(
      "Pro customers can input custom CSS for variations",
      "content-blocks-builder"
    );

    const selector =
      blockName === "core/html"
        ? ".cbb-html-preview"
        : `.wp-block-post-content > .wp-block-${blockName
            .replace("core/", "")
            .replace("/", "-")}`;

    const disabledAnimationCSS =
      ".is-selected, .has-child-selected, .has-child-selected * {animation: none !important;}";

    const styleControls = applyFilters(
      "boldblocks.manageVariations.customStyle",
      null,
      {
        customStyle,
        onChangeCustomStyle: (value) => {
          let formatedValue = refineCustomCSS(value, { selector });
          const isHTMLPreview = blockName === "core/html";
          if (formatedValue) {
            formatedValue = `${formatedValue}${disabledAnimationCSS}`;
            updateDynamicStyle(
              "edit-variation-style-inline-css",
              formatedValue,
              false,
              isHTMLPreview
            );
          } else {
            updateDynamicStyle(
              "edit-variation-style-inline-css",
              "",
              true,
              isHTMLPreview
            );
          }

          updateMeta("boldblocks_variation_custom_style")(value);
        },
      }
    );

    return (
      <>
        <PluginDocumentSettingPanel
          name="content-block-settings"
          title={pluginTitle}
          initialOpen={true}
        >
          {PluginControls}
          <NestedPanelBody title={manageStylesTitle} initialOpen={false}>
            {isPremium ? styleControls : disabledStylesTitle}
          </NestedPanelBody>
        </PluginDocumentSettingPanel>
        <PluginSidebarMoreMenuItem target="variation-settings">
          {pluginTitle}
        </PluginSidebarMoreMenuItem>
        <PluginSidebar name="variation-settings" title={pluginTitle}>
          <PanelBody>{PluginControls}</PanelBody>
          <PanelBody title={manageStylesTitle} initialOpen={false}>
            {isPremium ? styleControls : disabledStylesTitle}
          </PanelBody>
        </PluginSidebar>
      </>
    );
  },
});

registerPlugin("boldblocks-disable-variations-with-missing-blocks", {
  render: () => {
    const { createWarningNotice } = useDispatch(noticesStore);
    const { getMissingBlockStatus } = useSelect(
      (select) => select(boldblocksDataStore),
      []
    );
    const { loadMissingBlock, setMissingBlockStatus } = useDispatch(
      boldblocksDataStore
    );
    const canManagePlugins = userCanManagePlugins();

    const missingVariations = useMemo(() => {
      const availableBlockTypes = getBlockTypes();
      const availableBlockNames = map(availableBlockTypes, "name");

      return (window?.BoldBlocksVariations ?? []).reduce(
        (
          prev,
          { id, blockName, variationName, variationData, dependentBlocks }
        ) => {
          if (!isArray(dependentBlocks) || !dependentBlocks.length) {
            let variation;
            try {
              ({ variation } = JSON.parse(variationData));
            } catch (error) {
              log(error, "error");
            }

            if (variation && isObject(variation)) {
              const { innerBlocks = [] } = variation;
              dependentBlocks = getVariationDependentBlockTypes(
                innerBlocks,
                []
              );
            }
          }

          if (isArray(dependentBlocks) && !!dependentBlocks.length) {
            const missingBlock = hasMissingBlocks(
              dependentBlocks,
              availableBlockNames
            );
            if (missingBlock) {
              prev.push({
                id,
                variationName,
                blockName,
                missingBlock,
              });
            }
          }

          return prev;
        },
        []
      );
    }, []);

    useEffect(() => {
      if (!!missingVariations.length) {
        missingVariations.forEach((variation) => {
          const { blockName, variationName } = variation;
          unregisterBlockVariation(blockName, variationName);
        });
      }
    }, []);

    useEffect(() => {
      if (!isUndefined(canManagePlugins) && !!missingVariations.length) {
        missingVariations.forEach(async (variation) => {
          const { id, missingBlock, variationName: name } = variation;

          const noticeId = `variation-${id}`;
          if (!getMissingBlockStatus(noticeId)) {
            let actions = [
              {
                label: __("Edit variation", "content-blocks-builder"),
                url: getEditPostURL(id),
              },
            ];

            const noticeMessage = sprintf(
              __(
                "The variation %s has been disabled because it contains a missing block %s.",
                "content-blocks-builder"
              ),
              name,
              missingBlock
            );

            log(noticeMessage, "warn");

            const filterValue = `block:${missingBlock}`;

            let plugin;
            try {
              plugin = await loadMissingBlock(filterValue);
            } catch (error) {
              log(error, "error");
            }

            if (plugin && !isEmpty(plugin)) {
              actions.push(...getNoticeActions(plugin));
            }

            createWarningNotice(noticeMessage, { actions });
            setMissingBlockStatus(noticeId);
          }
        });
      }
    }, [canManagePlugins, missingVariations.length]);

    return null;
  },
});

const getVariationDependentBlockTypes = (innerBlocks, blockTypes = []) => {
  if (!innerBlocks.length) {
    return blockTypes;
  }

  for (let i = 0; i < innerBlocks.length; i++) {
    const [blockName, , nestedInnerBlocks = []] = innerBlocks[i];
    if (blockTypes.indexOf(blockName) === -1) {
      blockTypes.push(blockName);
    }

    blockTypes = getVariationDependentBlockTypes(nestedInnerBlocks, blockTypes);
  }

  return blockTypes;
};
