/**
 * External dependencies
 */
import classnames from "classnames";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { registerBlockVariation, serialize } from "@wordpress/blocks";
import apiFetch from "@wordpress/api-fetch";

/**
 * Internal dependencies
 */
import { getEditPostURL } from "../utils/posts";

export const ContentAttributes = [
  // Common fields
  "id",
  "content",
  "title",
  "text",
  "customText",
  "moreText",
  "buttonText",
  "description",
  "slug",
  "url",
  "src",
  "href",
  "linkTo",
  "placeholder",
  "caption",
  "alt",
  "label",
  "userId",
  "byline",

  // Gallery, image
  "ids",
  "images",
  "linkDestination",

  // Media text
  "mediaAlt",
  "mediaId",
  "mediaUrl",
  "mediaLink",
  "mediaType",

  // Pull quote
  "citation",
  "value",

  // Video
  "poster",
  "tracks",
];

/**
 * Build innerBlocks for registerBlockVariation
 *
 * @param {Array} innerBlocks
 * @returns
 */
export const buildInnerBlocksForVariantions = (innerBlocks) => {
  let refinedInnerBlocks = [];
  if (innerBlocks && innerBlocks.length > 0) {
    innerBlocks.forEach(({ name, innerBlocks, attributes }) => {
      refinedInnerBlocks.push([
        name,
        attributes,
        buildInnerBlocksForVariantions(innerBlocks),
      ]);
    });
  }

  return refinedInnerBlocks;
};

/**
 * Build the meta data for variations
 *
 * @param {Object}
 * @returns {Object}
 */
export const buildVariationData = ({
  blockName,
  variationName,
  title,
  block,
}) => {
  const { attributes = {}, innerBlocks = [] } = block ?? {};

  const variationData = {
    blockName,
    variation: {
      name: variationName,
      title,
      attributes,
      innerBlocks: buildInnerBlocksForVariantions(innerBlocks),
    },
  };

  return variationData;
};

/**
 * Create a new variation
 *
 * @param {Object}
 */
export const createVariation = ({
  variationName,
  blockName,
  title,
  description = "",
  iconString = "",
  block,
  updateBlockAttributes,
  createSuccessNotice,
  createErrorNotice,
}) => {
  // Get post content
  const variationContent = serialize([block]);

  // Build the meta data for the variation.
  const variationData = buildVariationData({
    blockName,
    variationName,
    title,
    block,
  });

  apiFetch({
    path: "boldblocks/v1/createVariation",
    method: "POST",
    data: {
      title,
      content: variationContent,
      status: "publish",
      meta: {
        boldblocks_variation_block_name: blockName,
        boldblocks_variation_name: variationName,
        boldblocks_variation_data: JSON.stringify(variationData),
        boldblocks_variation_icon: iconString,
        boldblocks_variation_description: description,
      },
    },
  })
    .then((json) => {
      const { success, data, post } = json;
      if (success) {
        createSuccessNotice(
          data ||
            __(
              "Your variation has been created successfully!",
              "content-blocks-builder"
            ),
          {
            type: "snackbar",
            actions: [
              {
                label: __("Edit variation", "content-blocks-builder"),
                url: getEditPostURL(post.id),
              },
            ],
          }
        );

        // Register new variation
        registerBlockVariation(blockName, variationData.variation);

        setTimeout(() => {
          updateBlockAttributes(block.clientId, {
            className: classnames(
              block?.attributes?.className,
              `is-style-${variationName.replace("/", "-")}`
            ),
          });
        }, 0);
      } else {
        createErrorNotice(data || "Request failed!", {
          type: "snackbar",
        });
      }
    })
    .catch((e) => {
      createErrorNotice(e.message, { type: "snackbar" });
    });
};
